import SLOPpy
import argparse
import os
import sys
import collections

def sloppy_run(file_conf=None ):

    print()
    print('SLOPpy v{0}'.format(SLOPpy.__version__))
    print()
    print('Python version in use:')
    print(sys.version)
    #if sys.version_info[0] == 3 and sys.version_info[1] > 7:
    #    print('WARNING MESSAGES SUPPRESSED!')
    #print()

    if file_conf is None:
        parser = argparse.ArgumentParser(prog='SLOPpy_Run', description='SLOPpy runner')
        parser.add_argument('config_file', type=str, nargs=1, help='config file')
    
        args = parser.parse_args()
        file_conf = args.config_file[0]

    config_in = SLOPpy.yaml_parser(file_conf)

    SLOPpy.pars_input(config_in)

    #return()

    print()
    """ creation of the pickle files """
    SLOPpy.prepare_datasets(config_in)

    #""" Retrieving the dictionary with the pipeline recipes """
    #pipeline = config_in['pipeline']

    """ Recipes must be performed in a given order... that's why we must use and ordered dictionary"""
    """ Each of the following recipes has to be performed on the whole spectrum """
    pipeline_common_routines = collections.OrderedDict()

    pipeline_common_routines['sky_correction'] = SLOPpy.compute_sky_correction

    #pipeline_common_routines['PCA_test01'] = SLOPpy.PCA_test01

    pipeline_common_routines['pca_preparation'] = SLOPpy.compute_pca_preparation
    pipeline_common_routines['sysrem_correction'] = SLOPpy.compute_sysrem_correction


    # molecfit version 1.5
    pipeline_common_routines['telluric_molecfit_v1_preparation'] = SLOPpy.compute_telluric_molecfit_v1_preparation
    pipeline_common_routines['telluric_molecfit_v1'] = SLOPpy.compute_telluric_molecfit_v1
    pipeline_common_routines['telluric_molecfit_v1_coadd'] = SLOPpy.compute_telluric_molecfit_v1_coadd

    # molecfit new version
    pipeline_common_routines['telluric_molecfit_preparation'] = SLOPpy.compute_telluric_molecfit_preparation
    pipeline_common_routines['telluric_molecfit'] = SLOPpy.compute_telluric_molecfit
    pipeline_common_routines['telluric_molecfit_coadd'] = SLOPpy.compute_telluric_molecfit_coadd

    pipeline_common_routines['telluric_template'] = SLOPpy.compute_telluric_template
    pipeline_common_routines['telluric_template_reference'] = SLOPpy.compute_telluric_template_reference
    pipeline_common_routines['telluric_template_alternative'] = SLOPpy.compute_telluric_template_alternative

    pipeline_common_routines['telluric_airmass_stellarRF'] = SLOPpy.compute_telluric_airmass_stellarRF
    pipeline_common_routines['telluric_airmass_reference_stellarRF'] = SLOPpy.compute_telluric_airmass_reference_stellarRF

    pipeline_common_routines['telluric_airmass_observerRF'] = SLOPpy.compute_telluric_airmass_observerRF
    pipeline_common_routines['telluric_airmass_berv_observerRF'] = SLOPpy.compute_telluric_airmass_berv_observerRF
    pipeline_common_routines['telluric_airmass_reference_observerRF'] = SLOPpy.compute_telluric_airmass_reference_observerRF
    pipeline_common_routines['telluric_airmass_berv_reference_observerRF'] = SLOPpy.compute_telluric_airmass_berv_reference_observerRF


    #pipeline_routines['telluric_obsolete_wyttenbach'] = SLOPpy.compute_telluric_obsolete_wyttenbach
    #pipeline_routines['telluric_airmass_observerRF_chunks'] = SLOPpy.compute_telluric_airmass_observerRF_chunks
    pipeline_common_routines['telluric_observerRF_skycalc'] = SLOPpy.compute_telluric_observerRF_skycalc

    pipeline_common_routines['differential_refraction'] = SLOPpy.compute_differential_refraction
    pipeline_common_routines['differential_refraction_update'] = SLOPpy.compute_differential_refraction_update

    pipeline_common_routines['check_differential_refraction'] = SLOPpy.check_differential_refraction
    pipeline_common_routines['write_differential_refraction'] = SLOPpy.write_differential_refraction

    pipeline_common_routines['interstellar_lines'] = SLOPpy.compute_interstellar_lines

    pipeline_common_routines['master_out'] = SLOPpy.compute_master_out

    pipeline_common_routines['clv_rm_models'] = SLOPpy.compute_clv_rm_models
    pipeline_common_routines['clv_rm_models_doubleprecision'] = SLOPpy.compute_clv_rm_models_doubleprecision
    pipeline_common_routines['transmission_spectrum_preparation'] = SLOPpy.compute_transmission_spectrum_preparation

    pipeline_common_routines['write_output_transmission'] = SLOPpy.write_output_transmission
    pipeline_common_routines['write_output_transmission_stellarRF'] = SLOPpy.write_output_transmission_stellarRF
    pipeline_common_routines['write_output_transmission_planetRF'] = SLOPpy.write_output_transmission_planetRF
    pipeline_common_routines['write_output_transmission_observerRF'] = SLOPpy.write_output_transmission_observerRF


    """ Legacy routines for testing purposes """
    pipeline_routines = collections.OrderedDict()


    #pipeline_routines['transmission_spectrum_planetRF'] = SLOPpy.compute_transmission_spectrum_planetRF
    #pipeline_routines['transmission_spectrum_observerRF'] = SLOPpy.compute_transmission_spectrum_observerRF
    #pipeline_routines['transmission_spectrum_stellarRF'] = SLOPpy.compute_transmission_spectrum_stellarRF
    #pipeline_routines['transmission_spectrum'] = SLOPpy.compute_transmission_spectrum

    #pipeline_routines['second_telluric_correction_on_transmission'] = SLOPpy.compute_second_telluric_correction_on_transmission

    #pipeline_routines['transmission_map'] = SLOPpy.compute_transmission_map
    #pipeline_routines['transmission_clv_rm_map'] = SLOPpy.compute_transmission_clv_rm_map


    """ Each of the following recipes has to be performed independently on each
    set of spectral lines """

    pipeline_lines_routines = collections.OrderedDict()

    """
    pipeline_lines_routines['transmission_spectrum_planetRF'] = SLOPpy.compute_transmission_spectrum_planetRF
    pipeline_lines_routines['transmission_spectrum_observerRF'] = SLOPpy.compute_transmission_spectrum_observerRF
    pipeline_lines_routines['transmission_spectrum_stellarRF'] = SLOPpy.compute_transmission_spectrum_stellarRF
    pipeline_lines_routines['transmission_spectrum'] = SLOPpy.compute_transmission_spectrum

    pipeline_lines_routines['second_telluric_correction_on_transmission'] = SLOPpy.compute_second_telluric_correction_on_transmission


    pipeline_lines_routines['transmission_map'] = SLOPpy.compute_transmission_map
    pipeline_lines_routines['transmission_clv_rm_map'] = SLOPpy.compute_transmission_clv_rm_map

    #pipeline_lines_routines['clv_rm_modelling'] = SLOPpy.compute_clv_rm_modelling
    """

    # ! NEW
    pipeline_lines_routines['quick_transmission'] = SLOPpy.compute_quick_transmission
    pipeline_lines_routines['clv_rm_models_lines'] = SLOPpy.compute_clv_rm_models_lines
    pipeline_lines_routines['transmission_mcmc'] = SLOPpy.compute_transmission_mcmc
    pipeline_lines_routines['transmission_mcmc_iterative'] = SLOPpy.compute_transmission_mcmc_iterative
    pipeline_lines_routines['transmission_binned_mcmc'] = SLOPpy.compute_transmission_binned_mcmc
    pipeline_lines_routines['transmission_binned_mcmc_iterative'] = SLOPpy.compute_transmission_binned_mcmc_iterative

    pipeline_lines_routines['transmission_spectrum_planetRF'] = SLOPpy.compute_transmission_spectrum_planetRF
    pipeline_lines_routines['transmission_spectrum_observerRF'] = SLOPpy.compute_transmission_spectrum_observerRF
    pipeline_lines_routines['transmission_spectrum_stellarRF'] = SLOPpy.compute_transmission_spectrum_stellarRF
    pipeline_lines_routines['transmission_spectrum'] = SLOPpy.compute_transmission_spectrum

    pipeline_lines_routines['transmission_spectrum_planetRF_iterative'] = SLOPpy.compute_transmission_spectrum_planetRF_iterative
    pipeline_lines_routines['transmission_spectrum_observerRF_iterative'] = SLOPpy.compute_transmission_spectrum_observerRF_iterative
    pipeline_lines_routines['transmission_spectrum_stellarRF_iterative'] = SLOPpy.compute_transmission_spectrum_stellarRF_iterative
    pipeline_lines_routines['transmission_spectrum_iterative'] = SLOPpy.compute_transmission_spectrum_iterative


    pipeline_lines_routines['transmission_spectrum_average_planetRF'] = SLOPpy.compute_transmission_spectrum_average_planetRF
    pipeline_lines_routines['transmission_spectrum_average_observerRF'] = SLOPpy.compute_transmission_spectrum_average_observerRF
    pipeline_lines_routines['transmission_spectrum_average_stellarRF'] = SLOPpy.compute_transmission_spectrum_average_stellarRF
    pipeline_lines_routines['transmission_spectrum_average'] = SLOPpy.compute_transmission_spectrum_average

    pipeline_lines_routines['transmission_spectrum_average_planetRF_iterative'] = SLOPpy.compute_transmission_spectrum_average_planetRF
    pipeline_lines_routines['transmission_spectrum_average_observerRF_iterative'] = SLOPpy.compute_transmission_spectrum_average_observerRF
    pipeline_lines_routines['transmission_spectrum_average_stellarRF_iterative'] = SLOPpy.compute_transmission_spectrum_average_stellarRF
    pipeline_lines_routines['transmission_spectrum_average_iterative'] = SLOPpy.compute_transmission_spectrum_average


    pipeline_lines_routines['transmission_lightcurve'] = SLOPpy.compute_transmission_lightcurve
    pipeline_lines_routines['transmission_lightcurve_average'] = SLOPpy.compute_transmission_lightcurve_average

    pipeline_lines_routines['spectra_lightcurve'] = SLOPpy.compute_spectra_lightcurve
    pipeline_lines_routines['spectra_lightcurve_average'] = SLOPpy.compute_spectra_lightcurve_average


    # TODO: to be updated to support single line(s) set
    """
    pipeline_lines_routines['transmission_clv_rm_correction_planetRF'] = SLOPpy.compute_transmission_clv_rm_correction_planetRF
    pipeline_lines_routines['transmission_clv_rm_correction_observerRF'] = SLOPpy.compute_transmission_clv_rm_correction_observerRF
    pipeline_lines_routines['transmission_clv_rm_correction_stellarRF'] = SLOPpy.compute_transmission_clv_rm_correction_stellarRF
    pipeline_lines_routines['transmission_clv_rm_correction'] = SLOPpy.compute_transmission_clv_rm_correction

    pipeline_lines_routines['transmission_clv_rm_average_planetRF'] = SLOPpy.compute_transmission_clv_rm_average_planetRF
    pipeline_lines_routines['transmission_clv_rm_average_observerRF'] = SLOPpy.compute_transmission_clv_rm_average_observerRF
    pipeline_lines_routines['transmission_clv_rm_average_stellarRF'] = SLOPpy.compute_transmission_clv_rm_average_stellarRF
    pipeline_lines_routines['transmission_clv_rm_average'] = SLOPpy.compute_transmission_clv_rm_average



    pipeline_lines_routines['spectra_lightcurve'] = SLOPpy.compute_spectra_lightcurve
    pipeline_lines_routines['spectra_lightcurve_average'] = SLOPpy.compute_spectra_lightcurve_average

    pipeline_lines_routines['excess_lightcurve'] = SLOPpy.compute_spectra_lightcurve
    pipeline_lines_routines['excess_lightcurve_average'] = SLOPpy.compute_spectra_lightcurve_average

    pipeline_lines_routines['spectra_lightcurve_clv_rm_correction'] = SLOPpy.compute_spectra_lightcurve_clv_rm_correction
    pipeline_lines_routines['spectra_lightcurve_average_clv_rm_correction'] = SLOPpy.compute_spectra_lightcurve_average_clv_rm_correction

    pipeline_lines_routines['excess_lightcurve_clv_rm_correction'] = SLOPpy.compute_spectra_lightcurve_clv_rm_correction
    pipeline_lines_routines['excess_lightcurve_average_clv_rm_correction'] = SLOPpy.compute_spectra_lightcurve_average_clv_rm_correction



    pipeline_lines_routines['transmission_lightcurve_planetRF'] = SLOPpy.compute_transmission_lightcurve_planetRF
    pipeline_lines_routines['transmission_lightcurve_observerRF'] = SLOPpy.compute_transmission_lightcurve_observerRF
    pipeline_lines_routines['transmission_lightcurve_stellarRF'] = SLOPpy.compute_transmission_lightcurve_stellarRF
    pipeline_lines_routines['transmission_lightcurve'] = SLOPpy.compute_transmission_lightcurve

    pipeline_lines_routines['write_output_spectra'] = SLOPpy.write_output_spectra

    pipeline_lines_routines['transmission_lightcurve_average_planetRF'] = SLOPpy.compute_transmission_lightcurve_average_planetRF
    pipeline_lines_routines['transmission_lightcurve_average_observerRF'] = SLOPpy.compute_transmission_lightcurve_average_observerRF
    pipeline_lines_routines['transmission_lightcurve_average_stellarRF'] = SLOPpy.compute_transmission_lightcurve_average_stellarRF
    pipeline_lines_routines['transmission_lightcurve_average'] = SLOPpy.compute_transmission_lightcurve_average
    """


    plot_preparation_routines = collections.OrderedDict()
    #plot_preparation_routines['clv_rm_modelling'] = SLOPpy.plot_clv_rm_modelling

    # ! New
    plot_preparation_routines['clv_rm_models'] = SLOPpy.plot_clv_rm_models
    plot_preparation_routines['clv_rm_models_doubleprecision'] = SLOPpy.plot_clv_rm_models


    plot_routines = collections.OrderedDict()

    plot_routines['plot_dataset'] = SLOPpy.plot_dataset
    plot_routines['prepare_dataset'] = SLOPpy.plot_dataset
    plot_routines['dataset'] = SLOPpy.plot_dataset
    plot_routines['sky_correction'] = SLOPpy.plot_sky_correction

    plot_routines['differential_refraction'] = SLOPpy.plot_differential_refraction
    plot_routines['differential_refraction_update'] = SLOPpy.plot_differential_refraction_update

    plot_routines['check_differential_refraction'] = SLOPpy.plot_check_differential_refraction
    #plot_routines['write_differential_refraction'] = SLOPpy.write_differential_refraction

    #plot_routines['PCA_test01'] = SLOPpy.PCA_test01

    # molecfit version 1.5
    plot_routines['telluric_molecfit_v1'] = SLOPpy.plot_telluric_molecfit_v1
    plot_routines['telluric_molecfit_v1_coadd'] = SLOPpy.plot_telluric_molecfit_v1_coadd

    # molecfit new version
    plot_routines['telluric_molecfit'] = SLOPpy.plot_telluric_molecfit
    plot_routines['telluric_molecfit_coadd'] = SLOPpy.plot_telluric_molecfit_coadd

    plot_routines['telluric_template'] = SLOPpy.plot_telluric_template
    plot_routines['telluric_template_reference'] = SLOPpy.plot_telluric_template_reference
    plot_routines['telluric_template_alternative'] = SLOPpy.plot_telluric_template_alternative

    plot_routines['telluric_airmass_stellarRF'] = SLOPpy.plot_telluric_airmass_stellarRF
    plot_routines['telluric_airmass_reference_stellarRF'] = SLOPpy.plot_telluric_airmass_reference_stellarRF

    plot_routines['telluric_airmass_observerRF'] = SLOPpy.plot_telluric_airmass_observerRF
    plot_routines['telluric_airmass_berv_observerRF'] = SLOPpy.plot_telluric_airmass_berv_observerRF
    plot_routines['telluric_airmass_reference_observerRF'] = SLOPpy.plot_telluric_airmass_reference_observerRF
    plot_routines['telluric_airmass_berv_reference_observerRF'] = SLOPpy.plot_telluric_airmass_berv_reference_observerRF

    #plot_routines['telluric_obsolete_wyttenbach'] = SLOPpy.plot_telluric_obsolete_wyttenbach
    #plot_routines['telluric_airmass_observerRF_chunks'] = SLOPpy.plot_telluric_airmass_observerRF_chunks
    plot_routines['telluric_observerRF_skycalc'] = SLOPpy.plot_telluric_observerRF_skycalc

    plot_routines['interstellar_lines'] = SLOPpy.plot_interstellar_lines

    plot_routines['master_out'] = SLOPpy.plot_master_out

    plot_routines['telluric_molecfit_preparation'] = SLOPpy.plot_telluric_molecfit_preparation

    # ! NEW
    plot_routines['transmission_spectrum_preparation'] = SLOPpy.plot_transmission_spectrum_preparation

    """
    plot_routines['transmission_spectrum_planetRF'] = SLOPpy.plot_transmission_spectrum_planetRF
    plot_routines['transmission_spectrum_observerRF'] = SLOPpy.plot_transmission_spectrum_observerRF
    plot_routines['transmission_spectrum_stellarRF'] = SLOPpy.plot_transmission_spectrum_stellarRF
    plot_routines['transmission_spectrum'] = SLOPpy.plot_transmission_spectrum

    plot_routines['second_telluric_correction_on_transmission'] = SLOPpy.plot_second_telluric_correction_on_transmission


    plot_routines['transmission_clv_rm_correction_planetRF'] = SLOPpy.plot_transmission_clv_rm_correction_planetRF
    plot_routines['transmission_clv_rm_correction_observerRF'] = SLOPpy.plot_transmission_clv_rm_correction_observerRF
    plot_routines['transmission_clv_rm_correction_stellarRF'] = SLOPpy.plot_transmission_clv_rm_correction_stellarRF
    plot_routines['transmission_clv_rm_correction'] = SLOPpy.plot_transmission_clv_rm_correction


    plot_routines['spectra_lightcurve'] = SLOPpy.plot_spectra_lightcurve
    plot_routines['excess_lightcurve'] = SLOPpy.plot_spectra_lightcurve

    plot_routines['spectra_lightcurve_clv_rm_correction'] = SLOPpy.plot_spectra_lightcurve_clv_rm_correction
    plot_routines['excess_lightcurve_clv_rm_correction'] = SLOPpy.plot_spectra_lightcurve_clv_rm_correction


    plot_routines['transmission_lightcurve_planetRF'] = SLOPpy.plot_transmission_lightcurve_planetRF
    plot_routines['transmission_lightcurve_observerRF'] = SLOPpy.plot_transmission_lightcurve_observerRF
    plot_routines['transmission_lightcurve_stellarRF'] = SLOPpy.plot_transmission_lightcurve_stellarRF
    plot_routines['transmission_lightcurve'] = SLOPpy.plot_transmission_lightcurve

    plot_routines['transmission_map'] = SLOPpy.plot_transmission_map
    plot_routines['transmission_clv_rm_map'] = SLOPpy.plot_transmission_clv_rm_map
    """

    plot_lines_routines = collections.OrderedDict()
    plot_lines_routines['clv_rm_models_lines'] = SLOPpy.plot_clv_rm_models_lines


    plot_lines_routines['transmission_binned_mcmc'] = SLOPpy.plot_transmission_binned_mcmc

    plot_lines_routines['transmission_spectrum_planetRF'] = SLOPpy.plot_transmission_spectrum_planetRF
    plot_lines_routines['transmission_spectrum_observerRF'] = SLOPpy.plot_transmission_spectrum_observerRF
    plot_lines_routines['transmission_spectrum_stellarRF'] = SLOPpy.plot_transmission_spectrum_stellarRF
    plot_lines_routines['transmission_spectrum'] = SLOPpy.plot_transmission_spectrum

    plot_lines_routines['transmission_spectrum_planetRF_iterative'] = SLOPpy.plot_transmission_spectrum_planetRF_iterative
    plot_lines_routines['transmission_spectrum_observerRF_iterative'] = SLOPpy.plot_transmission_spectrum_observerRF_iterative
    plot_lines_routines['transmission_spectrum_stellarRF_iterative'] = SLOPpy.plot_transmission_spectrum_stellarRF_iterative
    plot_lines_routines['transmission_spectrum_iterative'] = SLOPpy.plot_transmission_spectrum_iterative





    plot_average_routines = collections.OrderedDict()
    plot_average_routines['compare_master_out'] = SLOPpy.plot_compare_master_out


    plot_lines_average_routines = collections.OrderedDict()

    # ! These should be removed and performed line by line !

    plot_lines_average_routines['transmission_binned_mcmc'] = SLOPpy.plot_transmission_binned_mcmc
    plot_lines_average_routines['transmission_spectrum_average_planetRF'] = SLOPpy.plot_transmission_spectrum_average_planetRF
    plot_lines_average_routines['transmission_spectrum_average_observerRF'] = SLOPpy.plot_transmission_spectrum_average_observerRF
    plot_lines_average_routines['transmission_spectrum_average_stellarRF'] = SLOPpy.plot_transmission_spectrum_average_stellarRF
    plot_lines_average_routines['transmission_spectrum_average'] = SLOPpy.plot_transmission_spectrum_average


    """
    plot_average_routines['excess_lightcurve_average'] = SLOPpy.plot_spectra_lightcurve_average
    plot_average_routines['spectra_lightcurve_average'] = SLOPpy.plot_spectra_lightcurve_average

    plot_average_routines['spectra_lightcurve_average_clv_rm_correction'] = \
        SLOPpy.plot_spectra_lightcurve_average_clv_rm_correction
    plot_average_routines['excess_lightcurve_average_clv_rm_correction'] = \
        SLOPpy.plot_spectra_lightcurve_average_clv_rm_correction

    plot_average_routines['transmission_average_planetRF'] = SLOPpy.plot_transmission_average_planetRF
    plot_average_routines['transmission_average_observerRF'] = SLOPpy.plot_transmission_average_observerRF
    plot_average_routines['transmission_average_stellarRF'] = SLOPpy.plot_transmission_average_stellarRF
    plot_average_routines['transmission_average'] = SLOPpy.plot_transmission_average

    plot_average_routines['transmission_lightcurve_average_planetRF'] = SLOPpy.plot_transmission_lightcurve_average_planetRF
    plot_average_routines['transmission_lightcurve_average_observerRF'] = SLOPpy.plot_transmission_lightcurve_average_observerRF
    plot_average_routines['transmission_lightcurve_average_stellarRF'] = SLOPpy.plot_transmission_lightcurve_average_stellarRF
    plot_average_routines['transmission_lightcurve_average'] = SLOPpy.plot_transmission_lightcurve_average

    plot_average_routines['transmission_clv_rm_average_planetRF'] = SLOPpy.plot_transmission_clv_rm_average_planetRF
    plot_average_routines['transmission_clv_rm_average_observerRF'] = SLOPpy.plot_transmission_clv_rm_average_observerRF
    plot_average_routines['transmission_clv_rm_average_stellarRF'] = SLOPpy.plot_transmission_clv_rm_average_stellarRF
    plot_average_routines['transmission_clv_rm_average'] = SLOPpy.plot_transmission_clv_rm_average

    plot_average_routines['compare_clv_rm_effects_planetRF'] = SLOPpy.plot_compare_clv_rm_effects_planetRF
    plot_average_routines['compare_clv_rm_effects_observerRF'] = SLOPpy.plot_compare_clv_rm_effects_observerRF
    plot_average_routines['compare_clv_rm_effects_stellarRF'] = SLOPpy.plot_compare_clv_rm_effects_stellarRF
    plot_average_routines['compare_clv_rm_effects'] = SLOPpy.plot_compare_clv_rm_effects

    plot_average_routines['transmission_map_average'] = SLOPpy.plot_transmission_map_average
    plot_average_routines['transmission_clv_rm_map_average'] = SLOPpy.plot_transmission_clv_rm_map_average

    """

    """
    Execution of subroutines
    """

    # ! NEW !
    print()
    print("*** Data preparation analysis ***")

    try:
        pipeline = config_in['pipeline']
        has_plots = len(pipeline)
    except (KeyError, TypeError):
        pipeline = {}


    for key in pipeline:
        if key in pipeline_common_routines:
            print()
            pipeline_common_routines[key](config_in)


    # ! Kept here for legacy purposes !
    for key in pipeline:
        if key in pipeline_routines:
            print()
            pipeline_routines[key](config_in)

    # ! NEW !
    print()
    print("*** Spectral lines analysis  ***")


    try:
        spectral_lines = config_in['spectral_lines']
        has_plots = len(spectral_lines)
    except (KeyError, TypeError):
        pipeline = {}

    for lines_label in spectral_lines:
        for key in pipeline:
            if key in pipeline_lines_routines:
                print()
                pipeline_lines_routines[key](config_in, lines_label)

    #for key, func in pipeline_routines.items():
    #    if key in pipeline: func(config_in)

    #TODO: must be updated to be performed on a single set of spectral lines

    try:
        plots = config_in['plots']
        has_plots = len(plots)
    except (KeyError, TypeError):
        return

    print()
    print("*** Plot Subroutines ***")
    print()

    plots = config_in['plots']
    nights = config_in['nights']

    for key in plots:
        if key in plot_preparation_routines:
            plot_preparation_routines[key](config_in)
            print()

    for key in plots:
        if key in plot_routines:
            plot_routines[key](config_in)
            print()

    for lines_label in config_in['spectral_lines']:

        for key in plots:

            for night in nights:
                    if key in plot_lines_routines:
                        plot_lines_routines[key](config_in, lines_label, night)
                        print()

            if key in plot_lines_average_routines:
                plot_lines_average_routines[key](config_in, lines_label)
                print()

        #for key, func in plot_preparation_routines.items():
        #    if key in plots: func(config_in)
        #
        #for night in nights:
        #    for key, func in plot_routines.items():
        #        if key in plots: func(config_in, night)
        #
        #for key, func in plot_average_routines.items():
        #    if key in plots: func(config_in)



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.plot_subroutines import *
from SLOPpy.subroutines.shortcuts import *
from SLOPpy.telluric_molecfit_preparation import compute_telluric_molecfit_preparation

__all__ = ["compute_telluric_molecfit_coadd",
           "plot_telluric_molecfit_coadd",
           "compute_telluric_molecfit",
           "plot_telluric_molecfit"]

subroutine_name = 'telluric_molecfit_coadd'


def compute_telluric_molecfit_coadd(config_in, no_coadding=False):

    night_dict = from_config_get_nights(config_in)
    instrument_dict = from_config_get_instrument(config_in)
    molecfit_dict = from_config_get_molecfit(config_in)

    compute_telluric_molecfit_preparation(config_in)

    aer_version = molecfit_dict.get('aer_version', '3.8.1.2')

    if no_coadding==True:
        print()
        print("--- WARNING ---: MOLECFIT model will be computed on individual spectra rather than on coadded spectra ")
        print("                 This task is accomplished by the same routine that computes the model on coadded spectra")
        print("                 With the only difference that the coadded spectra is made by a single observations")
        print("                 This choice proves a lot the maintenance of the code. As a consequence you will see a lot")
        print("                 of 'coadded' messages while running the code. ")
        print()
        molecfit_dict['exptime_coadd'] = 0.001
        subroutine_name = 'telluric_molecfit'
    else:
        subroutine_name = 'telluric_molecfit_coadd'



    for night in night_dict:

        instrument_name = night_dict[night]['instrument']
        template_dict = instrument_dict[instrument_name]['telluric_template']

        try:
            telluric = load_from_cpickle('telluric', config_in['output'], night)
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Retrieved'))
            continue
        except:
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Computing'))
            print()

        print('    instrument :', instrument_name)
        print()

        tellprep = load_from_cpickle('telluric_molecfit_preparation', config_in['output'], night)

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        """ Retrieving the observations"""
        calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
        input_data = retrieve_observations(config_in['output'], night, lists['observations'], use_telluric=False)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        processed = {
            'subroutine': 'telluric_molecfit',
            'n_orders': 0,
            'n_pixels': 0,
        }

        telluric = {
            'subroutine': 'telluric_molecfit',
            'reference_frame': 'observer'
        }

        processed['airmass_ref'] = 0.000
        processed['telluric'] = {}
        processed['rebin'] = {}
        processed['work_dir'] = tellprep['work_dir']

        """
        Molecfit works on pixel grid, so we must ensure that the spectra are rebinned always on the same wavelength
        scale and same wavelength step. We use local arrays for this purpose
        """

        processed['rebin']['wave'] = np.arange(input_data['coadd']['wavelength_range'][0],
                                               input_data['coadd']['wavelength_range'][1],
                                               molecfit_dict['rebinning_step'],
                                               dtype=np.double)

        processed['rebin']['size'] = np.size(processed['rebin']['wave'])
        processed['rebin']['step'] = np.ones(processed['rebin']['size'],
                                             dtype=np.double) * molecfit_dict['rebinning_step']

        processed['rebin'] = {
            'wave': input_data['coadd']['wave'],
            'size': input_data['coadd']['size'],
            'step': input_data['coadd']['step'],
        }

        # TODO: fix the wave:include files
        wave_include = '"'
        for wli_s, wli_e in zip(tellprep['include']['vacuum'][:, 0], tellprep['include']['vacuum'][:, 1]):
            wave_include = wave_include+str(wli_s)+','+str(wli_e)+','
        wave_include = wave_include[:-1]+'"'

        n_coadd = 0
        n_reference = 0
        texp_cumulated = 0.00
        texp_total = 0.000
        coadd_list = []

        # Computing the total integration time
        for n_obs, obs in enumerate(lists['observations']):
            texp_total += input_data[obs]['EXPTIME']

        print('  Writing data and configuration files for molecfit+calctrans')
        print()

        # There must be a more elegant way to do this, but I'm, not aware of it
        for n_obs, obs in enumerate(lists['observations']):

            input_data[obs]['molecfit']['aer_version'] = aer_version

            processed[obs] = {
                'n_orders': input_data[obs]['n_orders'],
                'n_pixels': input_data[obs]['n_pixels']
            }

            """ e2ds spectra are rescaled and then rebinned while keeping them in the Observer Reference Frame"""

            processed[obs]['e2ds_rescaling'], processed[obs]['e2ds_rescaled'], processed[obs]['e2ds_rescaled_err'] = \
                perform_rescaling(input_data[obs]['wave'],
                                  input_data[obs]['e2ds'],
                                  input_data[obs]['e2ds_err'],
                                  observational_pams['wavelength_rescaling'])

            preserve_flux = input_data[obs].get('absolute_flux', True)

            processed[obs]['rebin_ORF'] = \
                rebin_2d_to_1d(input_data[obs]['wave'],
                               input_data[obs]['step'],
                               processed[obs]['e2ds_rescaled'],
                               calib_data['blaze'],
                               processed['rebin']['wave'],
                               processed['rebin']['step'],
                               preserve_flux=preserve_flux,
                               rv_shift=0.00)

            """ This part is relative to the coadded spectrum, must be placed here because
                some variables such as directory names must be defined before the next step
                spectra are coadded to increase the SNR of the spectrum analyzed by molecfit
            """
            if n_coadd == 0:

                reference_name = 'coadded_{0:03d}'.format(n_reference)
                reference_dirname = './' + processed['work_dir'] + '/' + reference_name + '/'
                os.system('mkdir -p ' + reference_dirname)

                rebin_coadd = processed[obs]['rebin_ORF'].copy()

                molecfit_pams = {
                    'MJD': input_data[obs]['MJD'],
                    'UTC': input_data[obs]['UTC'],
                    'ELEVATION': input_data[obs]['ELEVATION'],
                    'HUMIDITY': input_data[obs]['HUMIDITY'],
                    'PRESSURE': input_data[obs]['PRESSURE'],
                    'TEMPERATURE_EN': input_data[obs]['TEMPERATURE_EN'],
                    'TEMPERATURE_M1': input_data[obs]['TEMPERATURE_M1'],
                    'AIRM_START': input_data[obs]['AIRM_START'],
                    'AIRM_END': input_data[obs]['AIRM_END']}

                coadded_files = open(reference_dirname + reference_name + '_files.list', 'w')

                coadd_list.append(reference_name)
                observations_dirlist = []
                observations_exelist = []

            else:
                rebin_coadd += processed[obs]['rebin_ORF']

                molecfit_pams['MJD'] += input_data[obs]['MJD']
                molecfit_pams['UTC'] += input_data[obs]['UTC']
                molecfit_pams['ELEVATION'] += input_data[obs]['ELEVATION']
                molecfit_pams['HUMIDITY'] += input_data[obs]['HUMIDITY']
                molecfit_pams['PRESSURE'] += input_data[obs]['PRESSURE']
                molecfit_pams['TEMPERATURE_EN'] += input_data[obs]['TEMPERATURE_EN']
                molecfit_pams['TEMPERATURE_M1'] += input_data[obs]['TEMPERATURE_M1']
                molecfit_pams['AIRM_END'] = input_data[obs]['AIRM_END']

            n_coadd += 1
            coadded_files.write(obs + '\n')

            texp_cumulated += input_data[obs]['EXPTIME']

            # """ Molecfit analysis is skipped if the telluric correction has been computed already"""
            # if os.path.isfile('./molecfit_'+night +'/output/'+obs+'_ORF_s1d_TAC.dat'):
            # print('    molecfit+calctrans results for ' + obs + ' already available')
            # continue

            """
                This is the directory for MOLECFIT_CALCTRANS and MOLECFIT_CORRECT,
                which is different from the one where the coadded spectrum is saved
            """
            observation_dirname = './' + processed['work_dir'] + '/' + 'obs_{0:03d}'.format(n_obs) + '/'
            os.system('mkdir -p ' + observation_dirname)

            """ the spectrum is saved as a BinTable Fits file in a format suitable for molecfit
                this is the spectrum for MOLECFIT_CALCTRANS and MOLECFIT_CORRECT, so it is saved inside
                the folder with the observation name
            """
            observation_name = obs
            observation_tabname = obs + '_ORF_s1d.fits'

            """ HDU keywords required starting from Molecfit 4.3"""

            hdu_keywords = {
                'MJD-OBS': (input_data[obs]['MJD'], 'MJD-OBS'),
                'ESO OBS EXECTIME': (input_data[obs]['EXPTIME'], 'EXPTIME'), #not sure about this...
                'ESO TEL AIRM START': (input_data[obs]['AIRM_START'], 'AIRMASS START'),
                'ESO TEL AIRM END': (input_data[obs]['AIRM_END'], 'AIRMASS END')
            }

            write_molecfit_input_spectrum(processed['rebin']['wave'],
                                          processed[obs]['rebin_ORF'],
                                          observation_dirname + observation_tabname,
                                          keywords = hdu_keywords)

            observation_calctrans_parname = observation_name + '_calctrans.rc'
            write_calctrans_par(observation_dirname + observation_calctrans_parname)

            """ Writing the SOF files for MOLECFIT_CALCTRANS and MOLECFIT_CORRECT
                For the observed spectrum
            """
            observation_calctrans_sofname = obs + '_calctrans.sof'

            observation_calctrans_soffile = open(observation_dirname + observation_calctrans_sofname, 'w')
            observation_calctrans_soffile.write(observation_tabname+' SCIENCE\n')
            observation_calctrans_soffile.write('../' + reference_name + '/MODEL_MOLECULES.fits MODEL_MOLECULES\n')
            observation_calctrans_soffile.write('../' + reference_name + '/ATM_PARAMETERS.fits ATM_PARAMETERS\n')
            observation_calctrans_soffile.write(
                '../' + reference_name + '/BEST_FIT_PARAMETERS.fits BEST_FIT_PARAMETERS\n')
            observation_calctrans_soffile.close()

            """ Writing the bash script to execute MOLECFIT_CALCTRANS in the directory containing the science fits
            """
            bash_file = './' + processed['work_dir'] + '/calctrans_exec_' + obs + '.source'
            bash_script = open(bash_file, 'w')
            bash_script.write('#!/bin/bash \n')

            bash_script.write('export TMPDIR=$PWD\n')
            bash_script.write('echo  "   " executing calctrans on ' + obs + ' \n')
            bash_script.write('cd ' + observation_dirname + ' \n')

            bash_script.write(molecfit_dict['esorex_exec'] + ' --recipe-config=' + observation_calctrans_parname
                              + ' molecfit_calctrans ' + observation_calctrans_sofname + '> ' + obs + '_calctrans.log\n')
            bash_script.write('cd $TMPDIR \n')
            bash_script.close()

            observations_dirlist.append(observation_dirname)
            observations_exelist.append(bash_file)

            processed[obs]['dir_name'] = observation_dirname
            processed[obs]['tab_name'] = observation_tabname

            if (texp_cumulated >= molecfit_dict['exptime_coadd'] and
                    texp_total-texp_cumulated >= molecfit_dict['exptime_coadd']) \
                    or n_obs == len(lists['observations'])-1:

                coadded_files.close()
                print('   Coadded spectrum: ', n_reference)

                if os.path.exists(reference_dirname + 'TELLURIC_CORR.fits'):
                    print('      molecfit for ' + reference_name + ' previously completed')
                    print()
                else:

                    rebin_coadd /= n_coadd

                    """ the spectra is saved as a FITS file in a format suitable for molecfit """
                    reference_tabname = reference_name + '_ORF_s1d.fits'

                    """ New FITS keywords required by Molecfit 4.3"""
                    hdu_keywords = {
                        'MJD-OBS': (molecfit_pams['MJD']/n_coadd, 'MJD-OBS'),
                        'ESO OBS EXECTIME': (texp_cumulated, 'EXPTIME'),
                        'ESO TEL AIRM START': (molecfit_pams['AIRM_START'], 'AIRMASS START'),
                        'ESO TEL AIRM END': (molecfit_pams['AIRM_END'], 'AIRMASS END')
                    }

                    write_molecfit_input_spectrum(processed['rebin']['wave'],
                                                    rebin_coadd,
                                                    reference_dirname + reference_tabname,
                                                    keywords=hdu_keywords)

                    """ Average of the observational parameters """
                    for key in molecfit_pams:
                        molecfit_pams[key] /= n_coadd

                    molecfit_pams['GEOELEV'] = input_data[obs]['GEOELEV']
                    molecfit_pams['GEOLONG'] = input_data[obs]['GEOLONG']
                    molecfit_pams['GEOLAT'] = input_data[obs]['GEOLAT']

                    reference_molecfit_parname = reference_name + '_molecfit.rc'
                    write_molecfit_par(reference_dirname + reference_molecfit_parname,
                                        wave_include,
                                        input_data[obs]['molecfit'],
                                        molecfit_pams)

                    reference_calctrans_parname = reference_name + '_calctrans.rc'
                    write_calctrans_par(reference_dirname + reference_calctrans_parname)

                    reference_molecfit_sofname = reference_name + '_molecfit.sof'

                    reference_molecfit_soffile = open(reference_dirname + reference_molecfit_sofname, 'w')
                    reference_molecfit_soffile.write(reference_tabname + ' SCIENCE\n')
                    reference_molecfit_soffile.close()

                    """ Writing the SOF files for MOLECFIT_CALCTRANS and MOLECFIT_CORRECT
                        For the observed spectrum
                    """
                    reference_calctrans_sofname = obs + '_calctrans.sof'

                    reference_calctrans_soffile = open(reference_dirname + reference_calctrans_sofname, 'w')
                    reference_calctrans_soffile.write(reference_tabname+' SCIENCE\n')
                    reference_calctrans_soffile.write('MODEL_MOLECULES.fits MODEL_MOLECULES\n')
                    reference_calctrans_soffile.write('ATM_PARAMETERS.fits ATM_PARAMETERS\n')
                    reference_calctrans_soffile.write('BEST_FIT_PARAMETERS.fits BEST_FIT_PARAMETERS\n')
                    reference_calctrans_soffile.close()

                    """ Writing the bash script to execute MOLECFIT_MODEL and MOLECFIT_CALCTRANS in the directory containing the coadded fits
                    """

                    bash_file = './' + processed['work_dir'] + '/molecfit_exec_' + reference_name + '.source'
                    bash_script = open(bash_file, 'w')
                    bash_script.write('#!/bin/bash \n')

                    bash_script.write('export TMPDIR=$PWD\n')
                    bash_script.write('echo  "   " executing molecfit on ' + reference_name + ' \n')
                    bash_script.write('cd ' + reference_dirname + ' \n')

                    bash_script.write(molecfit_dict['esorex_exec'] + ' --recipe-config=' + reference_molecfit_parname
                                      + ' molecfit_model ' + reference_molecfit_sofname + '> ' + obs + '_molecfit.log\n')
                    bash_script.write(molecfit_dict['esorex_exec'] + ' --recipe-config=' + reference_calctrans_parname
                                      + ' molecfit_calctrans ' + reference_calctrans_sofname + '> ' + obs + '_calctrans.log\n')
                    bash_script.write('cd $TMPDIR \n')
                    bash_script.close()

                    os.system('. ' + bash_file)

                for dirname, exename in zip(observations_dirlist, observations_exelist):
                    if os.path.exists(dirname + 'TELLURIC_CORR.fits'):
                        print('      molecfit for ' + dirname + ' previously completed')
                        print()
                    else:
                        os.system('. ' + exename)

                n_coadd = 0
                n_reference += 1
                texp_total -= texp_cumulated
                texp_cumulated = 0.0

        print()

        for n_obs, obs in enumerate(lists['observations']):

            telluric[obs] = {}
            observation_dirname = processed[obs]['dir_name']

            print('  Telluric correction for ', obs, 'retrieved from ', observation_dirname + 'TELLURIC_CORR.fits')

            """ Loading the telluric spectrum from the output directory of molecfit """
            corr_fits = fits.open(observation_dirname + 'TELLURIC_CORR.fits')
            # orig_fits = fits.open(observation_dirname + observation_tabname)
            telluric_molecfit = corr_fits[1].data
            """ rebinning onto the e2ds wave scale"""

            if molecfit_dict.get('fix_telluric', True):
                print('      fix_telluric applied -  temporary workaround for line at 5885.97 A [ORF]')
                line_boundaries = [5885.74, 5886.21]
                sel = (processed['rebin']['wave'] > line_boundaries[0]) \
                    & (processed['rebin']['wave'] < line_boundaries[1])
                tell_cont = np.amax(telluric_molecfit[sel])

                telluric_molecfit[sel] = (telluric_molecfit[sel] - tell_cont) / 2.0 + tell_cont

            telluric[obs]['spectrum'] = \
                rebin_1d_to_2d(processed['rebin']['wave'],
                               processed['rebin']['step'],
                               telluric_molecfit,
                               input_data[obs]['wave'],
                               input_data[obs]['step'],
                               preserve_flux=False)

            try:
                telluric[obs]['spectrum'] = np.nan_to_num(nan=1.0, posinf=1.0, neginf=1.0)
            except:
                temp = ~(np.isfinite(telluric[obs]['spectrum']))
                telluric[obs]['spectrum'][temp] = 1.0
            sel = telluric[obs]['spectrum'] < 0.0001
            telluric[obs]['spectrum'][sel] = 1.0

            telluric[obs]['airmass'] = input_data[obs]['AIRMASS']

            " for compatibilty to some plots, even if it doesn't make any sense"
            telluric[obs]['airmass_ref'] = 0.000
            telluric[obs]['spectrum_noairmass'] = np.power(telluric[obs]['spectrum'],
                                                           telluric[obs]['airmass_ref'] - input_data[obs]['AIRMASS'])
            telluric[obs]['null'] = telluric[obs]['spectrum_noairmass'] < 0.001
            telluric[obs]['spectrum_noairmass'][telluric[obs]['null']] = 1.0
            # we just copy the spectrum file, it's it's a model itself
            telluric[obs]['spline'] = telluric[obs]['spectrum'].copy()

            processed[obs]['e2ds_corrected'] = processed[obs]['e2ds_rescaled'] / telluric[obs]['spectrum']
            processed[obs]['e2ds_corrected_err'] = processed[obs]['e2ds_rescaled_err'] / telluric[obs]['spectrum']

        save_to_cpickle('telluric', telluric, config_in['output'], night)
        save_to_cpickle('telluric_processed', processed, config_in['output'], night)

        print()
        print("Night ", night, " completed")


def plot_telluric_molecfit_coadd(config_in, night_input='', no_coadding=False):
    import matplotlib.pyplot as plt

    night_dict = from_config_get_nights(config_in)
    instrument_dict = from_config_get_instrument(config_in)
    system_dict = from_config_get_system(config_in)

    if no_coadding==True:
        print()
        print("--- WARNING ---: MOLECFIT model will be computed on individual spectra rather than on coadded spectra ")
        print("                 This task is accomplished by the same routine that computes the model on coadded spectra")
        print("                 With the only difference that the coadded spectra is made by a single observations")
        print("                 This choice proves a lot the maintenance of the code. As a consequence you will see a lot")
        print("                 of 'coadded' messages while running the code. ")
        print()
        subroutine_name = 'telluric_molecfit'
    else:
        subroutine_name = 'telluric_molecfit_coadd'

    if night_input == '':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)

    for night in night_list:

        # plt.scatter(rescaling_array, computed_std, c='C0', zorder=1)
        # plt.scatter(sel_factor, sel_stdev, c='C1', zorder=2)
        # plt.plot(rescaling_array, np.polyval(coeff, rescaling_array))
        # plt.plot(rescaling_array, 2*rescaling_array*coeff[0] + coeff[1] )
        # plt.plot()

        print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Plotting'))


        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        """ Retrieving the analysis"""
        try:
            processed = load_from_cpickle('telluric_processed', config_in['output'], night)
            telluric = load_from_cpickle('telluric', config_in['output'], night)
        except:
            print()
            print("No telluric correction, no plots")
            continue

        input_data = retrieve_observations(config_in['output'], night, lists['observations'], use_telluric=False)

        colors, cmap, line_colors = make_color_array(lists, observational_pams)

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(2, 2, width_ratios=[50, 1])

        ax1 = plt.subplot(gs[0, 0])
        ax2 = plt.subplot(gs[1, 0], sharex=ax1)
        cbax1 = plt.subplot(gs[:, 1])

        lift_spectrum = 0.25

        for i, obs in enumerate(lists['observations']):
            color_array = cmap(i / len(lists['observations']))

            for order in range(0, processed[obs]['n_orders']):

                if order == 0 and i == 0:
                    ax1.plot(input_data[obs]['wave'][order, :],
                             processed[obs]['e2ds_rescaled'][order, :],
                             c=color_array, lw=1, alpha=0.5, label='uncorrected')
                    ax1.scatter(input_data[obs]['wave'][order, :],
                                processed[obs]['e2ds_corrected'][order, :],
                                s=1, c=np.atleast_2d(color_array), label='corrected')
                else:
                    ax1.plot(input_data[obs]['wave'][order, :],
                             processed[obs]['e2ds_rescaled'][order, :],
                             c=color_array, lw=1, alpha=0.5)
                    ax1.scatter(input_data[obs]['wave'][order, :],
                                processed[obs]['e2ds_corrected'][order, :],
                                s=1, c=np.atleast_2d(color_array))

                # ax1.plot(processed[obs]['wave'][order, :],
                #            e2ds_rescaled[order, :]+lift_spectrum,
                #            c=color_array, lw=1, alpha=0.5)
                # ax1.scatter(processed[obs]['wave'][order, :],
                #            e2ds_rescaled_corrected_spline[order, :]+lift_spectrum,
                #            s=1, c=np.atleast_2d(color_array))

                ax2.plot(input_data[obs]['wave'][order, :],
                         telluric[obs]['spectrum'][order, :],
                         c=color_array)
                ax2.axhline(1.00, c='k')

                # ax2.plot(processed[obs]['wave'][order, :],
                #         telluric[obs]['spline'][order, :]+lift_spectrum,
                #         c=color_array)
                # ax2.axhline(1.00+lift_spectrum, c='k')

        # ax2.plot(input_data['coadd']['wave'],telluric['stellarRF']['spline_eval']+0.1,c='k')
        # ax2.scatter(input_data['coadd']['wave'],telluric['stellarRF']['spectrum']+0.1,c='r', s=2)

        ax1.legend(loc=3)
        ax1.set_title('Night: ' + night)

        ax2.set_xlabel('$\lambda$ [$\AA$]')

        try:
            instrument = night_dict[night]['instrument']
            comparison_file = config_in['instruments'][instrument]['telluric_comparison']
            comparison_data = np.genfromtxt(comparison_file, skip_header=1)
            if comparison_data[0, 0] < 1000.0:
                nm2Ang = 10.
            else:
                nm2Ang = 1.
            ax1.plot(comparison_data[:, 0]*nm2Ang, comparison_data[:, 1], c='C0', zorder=1000)
            ax2.plot(comparison_data[:, 0]*nm2Ang, comparison_data[:, 1], c='C0', zorder=1000)
        except:
            pass

        sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=colors[0], vmax=colors[-1]))
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()


def compute_telluric_molecfit(config_in):
    compute_telluric_molecfit_coadd(config_in, no_coadding=True)

def plot_telluric_molecfit(config_in, night_input=''):
    plot_telluric_molecfit_coadd(config_in, night_input=night_input, no_coadding=True)



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *

from SLOPpy.subroutines.plot_subroutines import *
from SLOPpy.subroutines.shortcuts import *

__all__ = ["compute_telluric_airmass_stellarRF",
           "plot_telluric_airmass_stellarRF",
           "compute_telluric_airmass_reference_stellarRF",
           "plot_telluric_airmass_reference_stellarRF"]

subroutine_name = 'telluric_airmass_stellarRF'


def compute_telluric_airmass_stellarRF(config_in):

    compute_telluric_stellarRF(config_in,
                                use_reference_airmass=False,
                                subroutine_name='telluric_airmass_stellarRF')


def compute_telluric_airmass_reference_stellarRF(config_in):
    compute_telluric_stellarRF(config_in,
                                use_reference_airmass=True,
                                subroutine_name='telluric_airmass_reference_stellarRF')


def plot_telluric_airmass_reference_stellarRF(config_in, night_input):
    """ Alias to simplify the configuration file"""
    plot_telluric_airmass_stellarRF(config_in, night_input)


def compute_telluric_stellarRF(config_in, **kwargs):

    night_dict = from_config_get_nights(config_in)

    for night in night_dict:

        print()
        print("compute_telluric_airmass_stellarRF         Night: ", night)

        try:
            telluric = load_from_cpickle('telluric', config_in['output'], night)
            continue
        except:
            print()
            print("  No telluric correction file found, computing now ")

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
        input_data = retrieve_observations(config_in['output'], night, lists['observations'], use_telluric=False)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        processed = {
            'subroutine': subroutine_name
        }

        telluric = {
            'subroutine': kwargs['subroutine_name'],
            'reference_frame': 'stellar'
        }

        """ Reference airmass for iterative correction of airmass"""
        if kwargs['use_reference_airmass']:
            airmass_temp = np.zeros(lists['n_transit_in'])
            for n_obs, obs in enumerate(lists['transit_in']):
                # This is to ensure that airmass, berv and rvc are associated to the correct spectra
                airmass_temp[n_obs] = input_data[obs]['AIRMASS']
            processed['airmass_ref'] = np.average(airmass_temp)
        else:
            processed['airmass_ref'] = 0.000


        # There must be a more elegant way to do this, but I'm, not aware of it
        for obs in lists['observations']:

            processed[obs] = {
                'n_orders': input_data[obs]['n_orders'],
                'n_pixels': input_data[obs]['n_pixels']
            }
            telluric[obs] = {}

            """ for plotting purpose only"""
            processed[obs]['wave'] = input_data[obs]['wave']
            processed[obs]['e2ds'] = input_data[obs]['e2ds']
            processed[obs]['e2ds_err'] = input_data[obs]['e2ds_err']

            processed[obs]['flux'] = input_data[obs]['e2ds']/calib_data['blaze']/input_data[obs]['step']
            processed[obs]['flux_err'] = input_data[obs]['e2ds_err']/calib_data['blaze']/input_data[obs]['step']

            preserve_flux = input_data[obs].get('absolute_flux', True)

            processed[obs]['flux_SRF'] = \
                rebin_2d_to_1d(input_data[obs]['wave'],
                               input_data[obs]['step'],
                               input_data[obs]['e2ds'],
                               calib_data['blaze'],
                               input_data['coadd']['wave'],
                               input_data['coadd']['step'],
                               preserve_flux=preserve_flux,
                               rv_shift=observational_pams[obs]['rv_shift_ORF2SRF_mod'])

            processed[obs]['flux_SRF_err'] = \
                rebin_2d_to_1d(input_data[obs]['wave'],
                               input_data[obs]['step'],
                               input_data[obs]['e2ds_err'],
                               calib_data['blaze'],
                               input_data['coadd']['wave'],
                               input_data['coadd']['step'],
                               preserve_flux=preserve_flux,
                               rv_shift=observational_pams[obs]['rv_shift_ORF2SRF_mod'],
                               is_error=True)

            """ Zero or negative values are identified, flagged and substituted with another value """
            processed[obs]['flux_SRF'], processed[obs]['flux_SRF_err'], processed[obs]['null'] = \
                replace_values_errors(processed[obs]['flux_SRF'], processed[obs]['flux_SRF_err'], 0.0001)

            """rescaling"""
            processed[obs]['flux_SRF_rescaling'], processed[obs]['flux_SRF_rescaled'], processed[obs]['flux_SRF_rescaled_err']  = \
                perform_rescaling(input_data['coadd']['wave'],
                                  processed[obs]['flux_SRF'],
                                  processed[obs]['flux_SRF_err'],
                                  observational_pams['wavelength_rescaling'])

            processed[obs]['logI'] = np.log(processed[obs]['flux_SRF_rescaled'])
            processed[obs]['logI_err'] = processed[obs]['flux_SRF_rescaled_err'] / processed[obs]['flux_SRF_rescaled']

        processed['telluric'] = {}

        n_coadd = np.size(input_data['coadd']['wave'])

        abs_slope = np.ones(n_coadd, dtype=np.double)
        line_shift = np.ones(n_coadd, dtype=np.double)
        zero_point = np.ones(n_coadd, dtype=np.double)
        pearson_r = np.zeros(n_coadd, dtype=np.double)
        pearson_p = np.zeros(n_coadd, dtype=np.double)

        airmass = np.zeros(lists['n_tellurics'], dtype=np.double)
        berv = np.zeros(lists['n_tellurics'], dtype=np.double)
        rvc = np.zeros(lists['n_tellurics'], dtype=np.double)

        for n_obs, obs in enumerate(lists['telluric']):
            # This is to ensure that airmass, berv and rvc are associated to the correct spectra
            processed['telluric'][obs] = {'n_obs': n_obs}
            airmass[n_obs] = observational_pams[obs]['AIRMASS']
            berv[n_obs] = observational_pams[obs]['BERV']
            rvc[n_obs] = observational_pams[obs]['RVC']

        logi_array = np.empty([lists['n_tellurics'], n_coadd], dtype=np.double)
        sigi_array = np.empty([lists['n_tellurics'], n_coadd], dtype=np.double)

        for obs in lists['telluric']:

            n_obs = processed['telluric'][obs]['n_obs']
            logi_array[n_obs, :] = processed[obs]['logI'][:]
            sigi_array[n_obs, :] = processed[obs]['logI_err'][:]

            """ The user has the option to select between different approaches to
                extract the telluric absorption spectrum
                To-Do: move this section to a subroutine for cythonization"""

        if observational_pams['linear_fit_method'] == 'linear_curve_fit':
            abs_slope, zero_point = \
                airmass_linear_curve_fit(airmass, logi_array, sigi_array, n_coadd)

        else:

            abs_slope, zero_point = \
                airmass_linear_lstsq(airmass, logi_array)

        telluric['stellarRF'] = {
            'wave': input_data['coadd']['wave'],
            'step': input_data['coadd']['step']
        }

        telluric['stellarRF']['spectrum'] = np.exp(abs_slope)

        telluric['stellarRF']['emission'] = (telluric['stellarRF']['spectrum'] > 1.00000)
        telluric['stellarRF']['spectrum_fixed'] = telluric['stellarRF']['spectrum'][:]
        telluric['stellarRF']['spectrum_fixed'][telluric['stellarRF']['emission']]= 1.000

        telluric['stellarRF']['spline_eval'], \
        telluric['stellarRF']['spline_coeff'], \
        telluric['stellarRF']['spline_knots'] = \
        compute_spline(input_data['coadd']['wave'],
                       telluric['stellarRF']['spectrum_fixed'],
                       0.05)

        telluric['airmass_ref'] = processed['airmass_ref']

        """ Moving the spline to the observerRF in the e2ds"""
        for obs in lists['observations']:
            """ 1) shifting the telluric correction spline to the observer RV"""

            wave_ORF = shift_wavelength_array(np.asarray(telluric['stellarRF']['spline_coeff'][0]),
                                              -observational_pams[obs]['rv_shift_ORF2SRF_mod'])

            """ 2) new spline coefficients """
            tck1 = [shift_wavelength_array(np.asarray(telluric['stellarRF']['spline_coeff'][0]),
                                              - observational_pams[obs]['rv_shift_ORF2SRF_mod']),
                    telluric['stellarRF']['spline_coeff'][1],
                    telluric['stellarRF']['spline_coeff'][2]]

            """ 3) computation of the spline at the location of the spectra, taking care of the regions 
            out of the coadded spectrum """

            inside_spline = (input_data[obs]['wave'] > wave_ORF[0]) & (input_data[obs]['wave'] < wave_ORF[-1])

            telluric[obs]['spline_noairmass'] = np.ones([input_data[obs]['n_orders'],
                                                         input_data[obs]['n_pixels']],
                                                        dtype=np.double)

            for order in range(0, input_data[obs]['n_orders']):

                if np.sum(inside_spline[order, :])>0 :
                    telluric[obs]['spline_noairmass'][order, inside_spline[order, :]] = \
                        sci_int.splev(input_data[obs]['wave'][order, inside_spline[order, :]], tck1)

            telluric[obs]['spline'] = np.power(telluric[obs]['spline_noairmass'],
                                               observational_pams[obs]['AIRMASS'] - processed['airmass_ref'])

            """ Now a similar approach is followed for the telluric spectrum before spline fit 
            """

            telluric[obs]['spectrum_noairmass'] = \
                rebin_1d_to_2d(input_data['coadd']['wave'],
                               input_data['coadd']['step'],
                               telluric['stellarRF']['spectrum'],
                               input_data[obs]['wave'],
                               input_data[obs]['step'],
                               rv_shift=-observational_pams[obs]['rv_shift_ORF2SRF_mod'],
                               preserve_flux=False)

            telluric[obs]['null'] = telluric[obs]['spectrum_noairmass'] < 0.001
            telluric[obs]['spectrum_noairmass'][ telluric[obs]['null']] = 1.0

            telluric[obs]['spectrum'] = np.power(telluric[obs]['spectrum_noairmass'],
                                               input_data[obs]['AIRMASS'] - processed['airmass_ref'])

            telluric[obs]['airmass'] = input_data[obs]['AIRMASS']
            telluric[obs]['airmass_ref'] = processed['airmass_ref']
            telluric[obs]['rv_shift_ORF2SRF_mod'] = observational_pams[obs]['rv_shift_ORF2SRF_mod']

        save_to_cpickle('telluric_processed', processed, config_in['output'], night)
        save_to_cpickle('telluric', telluric, config_in['output'], night)

        print()
        print("Night ", night, " completed")


def plot_telluric_airmass_stellarRF(config_in, night_input=''):
    import matplotlib.pyplot as plt

    night_dict = from_config_get_nights(config_in)
    instrument_dict = from_config_get_instrument(config_in)
    system_dict = from_config_get_system(config_in)

    if night_input == '':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)

    for night in night_list:

        print("plot_telluric_airmass_stellarRF            Night: ", night)

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        """ Retrieving the analysis"""
        try:
            processed = load_from_cpickle('telluric_processed', config_in['output'], night)
            telluric = load_from_cpickle('telluric', config_in['output'], night)
        except:
            print()
            print("No telluric correction, no plots")
            continue

        colors, cmap, line_colors = make_color_array(lists, observational_pams)

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(2, 2, width_ratios=[50, 1])

        ax1 = plt.subplot(gs[0, 0])
        ax2 = plt.subplot(gs[1, 0], sharex=ax1)
        cbax1 = plt.subplot(gs[:, 1])

        lift_spectrum = 0.25

        for i, obs in enumerate(lists['observations']):
            color_array = cmap(i / len(lists['observations']))

            _, e2ds_rescaled , _ = \
                perform_rescaling(processed[obs]['wave'],
                                  processed[obs]['e2ds'],
                                  processed[obs]['e2ds_err'],
                                  observational_pams['wavelength_rescaling'])

            e2ds_rescaled_corrected_spectrum = e2ds_rescaled / telluric[obs]['spectrum']
            e2ds_rescaled_corrected_spline = e2ds_rescaled / telluric[obs]['spline']

            for order in range(0, processed[obs]['n_orders']):

                if order == 0 and i==0:
                    ax1.plot(processed[obs]['wave'][order, :],
                             e2ds_rescaled[order, :],
                             c=color_array, lw=1, alpha=0.5, label='uncorrected')
                    ax1.scatter(processed[obs]['wave'][order, :],
                                e2ds_rescaled_corrected_spectrum[order, :],
                                s=1, c=np.atleast_2d(color_array), label='corrected')
                else:
                    ax1.plot(processed[obs]['wave'][order, :],
                                e2ds_rescaled[order, :],
                                c=color_array, lw=1, alpha=0.5)
                    ax1.scatter(processed[obs]['wave'][order, :],
                                e2ds_rescaled_corrected_spectrum[order, :],
                                s=1, c=np.atleast_2d(color_array))

                #ax1.plot(processed[obs]['wave'][order, :],
                #            e2ds_rescaled[order, :]+lift_spectrum,
                #            c=color_array, lw=1, alpha=0.5)
                #ax1.scatter(processed[obs]['wave'][order, :],
                #            e2ds_rescaled_corrected_spline[order, :]+lift_spectrum,
                #            s=1, c=np.atleast_2d(color_array))

                ax2.plot(processed[obs]['wave'][order, :],
                         telluric[obs]['spectrum'][order, :],
                         c=color_array)
                ax2.axhline(1.00, c='k')

                #ax2.plot(processed[obs]['wave'][order, :],
                #         telluric[obs]['spline'][order, :]+lift_spectrum,
                #         c=color_array)
                #ax2.axhline(1.00+lift_spectrum, c='k')

        #ax2.plot(input_data['coadd']['wave'],telluric['stellarRF']['spline_eval']+0.1,c='k')
        #ax2.scatter(input_data['coadd']['wave'],telluric['stellarRF']['spectrum']+0.1,c='r', s=2)

        ax1.legend(loc=3)
        ax1.set_title('Night: ' + night)

        ax2.set_xlabel('$\lambda$ [$\AA$]')

        sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=colors[0], vmax=colors[-1]))
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from SLOPpy.sloppy_run import sloppy_run
from SLOPpy.subroutines.io_subroutines import yaml_parser, pars_input
#from SLOPpy.subroutines.interpol import interpolate1d_grid_nocheck
from SLOPpy.prepare_datasets import *
from SLOPpy.sky_correction import *
from SLOPpy.differential_refraction_preparation import *
from SLOPpy.differential_refraction import *
from SLOPpy.check_differential_refraction import *
from SLOPpy.telluric_template import *
from SLOPpy.telluric_molecfit_v1_preparation import *
from SLOPpy.telluric_molecfit_v1_coadd import *
from SLOPpy.telluric_molecfit_preparation import *
from SLOPpy.telluric_molecfit_coadd import *
from SLOPpy.telluric_template_alternative import *
from SLOPpy.telluric_airmass_stellarRF import *
from SLOPpy.telluric_airmass_observerRF import *
from SLOPpy.telluric_airmass_observerRF_chunks import *
from SLOPpy.telluric_observerRF_skycalc import *
from SLOPpy.interstellar_lines import *
from SLOPpy.master_out import *
from SLOPpy.transmission_spectrum_preparation import *
from SLOPpy.transmission_spectrum import *
from SLOPpy.transmission_spectrum_average import *
from SLOPpy.transmission_spectrum_shortcuts import *
from SLOPpy.second_telluric_correction_on_transmission import *
#from SLOPpy.clv_rm_modelling import *
from SLOPpy.compare_clv_rm_effects import *
from SLOPpy.spectra_lightcurve import *
from SLOPpy.spectra_lightcurve_average import *
from SLOPpy.transmission_lightcurve import *
from SLOPpy.transmission_lightcurve_average import *
from SLOPpy.write_output_spectra import *
from SLOPpy.write_output_transmission import *

from SLOPpy.quick_transmission import *

# NEW
from SLOPpy.clv_rm_models import *
from SLOPpy.clv_rm_models_doubleprecision import *
from SLOPpy.clv_rm_models_lines import *
from SLOPpy.transmission_mcmc import *
from SLOPpy.transmission_binned_mcmc import *

from SLOPpy.pca_preparation import *
from SLOPpy.sysrem_correction import *

__version__ = "1.5.1"



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.shortcuts import *
from SLOPpy.subroutines.constants import *
from SLOPpy.subroutines.kepler_exo import *
from SLOPpy.subroutines.plot_subroutines import *

from astropy.convolution import Gaussian1DKernel, convolve

__all__ = ['compute_clv_rm_modelling', 'plot_clv_rm_modelling']

subroutine_name = 'clv_rm_modelling'

### OLD modelling without the dependency on the planetar yradius at a given wavelngth

def compute_clv_rm_modelling(config_in):
    night_dict = from_config_get_nights(config_in)
    instrument_dict = from_config_get_instrument(config_in)

    planet_dict = from_config_get_planet(config_in)
    star_dict = from_config_get_star(config_in)
    clv_rm_dict = from_config_get_clv_rm(config_in)

    try:
        synthesis = load_from_cpickle('clv_rm_synthesis', config_in['output'])
        star_grid = load_from_cpickle('clv_rm_star_grid', config_in['output'])

        if not config_in['settings'].get('full_output', False):
            for night in night_dict:
                clv_rm_modelling = load_from_cpickle('clv_rm_modelling', config_in['output'], night)

        print("{0:45s}                         {1:s}".format(subroutine_name, 'Retrieved'))
        return

    except:
        print("{0:45s}                         {1:s}".format(subroutine_name, 'Computing'))
        print()

        """
        Loading the spectral synthesis results, at the moment only SME output is supported.
        Properties of the synthesis data files
        - limb_angles: this is an input to SME, so it is specific on how the synthesis has been performed
        - spectra: stellar spectrum as a function of the limb angle, sampled near the spectral lines
        - model: integrated spectrum of the star
        """

        synthesis_data_limb_angles = np.genfromtxt(clv_rm_dict['synthesis_files'] + '_muvals.txt', dtype=np.double)
        synthesis_data_spectra = np.genfromtxt(clv_rm_dict['synthesis_files'] + '_spectra.txt', dtype=np.double)
        synthesis_data_model = np.genfromtxt(clv_rm_dict['synthesis_files'] + '_model.txt', dtype=np.double)

        synthesis = {
            'surface': {
                'wave': synthesis_data_spectra[:, 0],
                'flux': synthesis_data_spectra[:, 1:],
                'n_mu': np.size(synthesis_data_limb_angles),
                'mu': synthesis_data_limb_angles
            },
            'total': {
                'wave': synthesis_data_model[:, 0],
                'norm': synthesis_data_model[:, 1],
            }
        }

        """ Setting up the array for model computation """

        synthesis['total']['step'] = synthesis['total']['wave'] * 0.0
        synthesis['total']['step'][1:] = synthesis['total']['wave'][1:] - synthesis['total']['wave'][:-1]
        synthesis['total']['step'][0] = synthesis['total']['step'][1]

        synthesis['surface']['step'] = synthesis['surface']['wave'] * 0.0
        synthesis['surface']['step'][1:] = synthesis['surface']['wave'][1:] - synthesis['surface']['wave'][:-1]
        synthesis['surface']['step'][0] = synthesis['surface']['step'][1]

        synthesis['surface']['wave_out'] = np.arange(synthesis['surface']['wave'][0],
                                                     synthesis['surface']['wave'][-1],
                                                     clv_rm_dict['rebinning_step'])
        synthesis['surface']['size_out'] = np.size(synthesis['surface']['wave_out'], axis=0)
        synthesis['surface']['step_out'] = np.ones(synthesis['surface']['size_out']) * clv_rm_dict['rebinning_step']

        synthesis['total']['norm_out'] = rebin_1d_to_1d(synthesis['total']['wave'],
                                                        synthesis['total']['step'],
                                                        synthesis['total']['norm'],
                                                        synthesis['surface']['wave_out'],
                                                        synthesis['surface']['step_out'],
                                                        method='exact_flux',
                                                        preserve_flux=False)

        """  Check if the number of spectra corresponds to the number of limb angle values """
        if np.size(synthesis['surface']['flux'], axis=1) != synthesis['surface']['n_mu']:
            print('ERROR in loading the stellar spectra')

        """
        Setting up the grid of stellar spectra for the CLV and RM computation
        odd number of points to include the zero value
        """

        star_grid = {
            'n_grid': clv_rm_dict['n_gridpoints'],
            'half_grid': int((clv_rm_dict['n_gridpoints'] - 1) / 2)
        }

        """ Coordinates of the centers of each grid cell (add offset) """
        star_grid['xx'] = np.linspace(-1.0, 1.0, star_grid['n_grid'], dtype=np.double)
        star_grid['xc'], star_grid['yc'] = np.meshgrid(star_grid['xx'], star_grid['xx'], indexing='xy')
        # check the Note section of the wiki page of meshgrid
        # https://docs.scipy.org/doc/numpy/reference/generated/numpy.meshgrid.html

        """ Distance of each grid cell from the center of the stellar disk """
        star_grid['rc'] = np.sqrt(star_grid['xc'] ** 2 + star_grid['yc'] ** 2)
        star_grid['inside'] = star_grid['rc'] <= 1.0  # Must avoid negative numbers inside the square root
        star_grid['outside'] = star_grid['rc'] > 1.0  # Must avoid negative numbers inside the square root

        """ Determine the mu angle for each grid cell, as a function of radius. """
        star_grid['mu'] = np.zeros([star_grid['n_grid'], star_grid['n_grid']],
                                   dtype=np.double)  # initialization of the matrix with the mu values
        star_grid['mu'][star_grid['inside']] = np.sqrt(1. - star_grid['rc'][star_grid['inside']] ** 2)

        """  2.2 Determine the Doppler shift to apply to the spectrum of each grid cell, from Cegla+2016 """

        star_grid['x_ortho'] = star_grid['xc'] * np.cos(star_dict['lambda'][0] * deg2rad) \
                               - star_grid['yc'] * np.sin(
            star_dict['lambda'][0] * deg2rad)  # orthogonal distances from the spin-axis
        star_grid['y_ortho'] = star_grid['xc'] * np.sin(star_dict['lambda'][0] * deg2rad) \
                               + star_grid['yc'] * np.cos(star_dict['lambda'][0] * deg2rad)

        star_grid['r_ortho'] = np.sqrt(star_grid['x_ortho'] ** 2 + star_grid['y_ortho'] ** 2)
        star_grid['z_ortho'] = np.zeros([star_grid['n_grid'], star_grid['n_grid']],
                                        dtype=np.double)  # initialization of the matrix
        star_grid['z_ortho'][star_grid['inside']] = np.sqrt(1 - star_grid['r_ortho'][star_grid['inside']] ** 2)

        """ rotate the coordinate system around the x_ortho axis by an angle: """
        star_grid['beta'] = (np.pi / 2.) - star_dict['inclination'][0] * deg2rad

        """ orthogonal distance from the stellar equator """
        star_grid['yp_ortho'] = star_grid['z_ortho'] * np.sin(star_grid['beta']) + star_grid['y_ortho'] * np.cos(
            star_grid['beta'])

        """ stellar rotational velocity for a given position """
        star_grid['v_star'] = star_grid['x_ortho'] * star_dict['vsini'][0] * (
                    1 - star_dict['alpha'][0] * star_grid['yp_ortho'] ** 2)
        star_grid['v_star'][star_grid['outside']] = 0.0  # Null velocity for points outside the stellar surface

        """ Associate a synthetic spectrum to each cell """

        star_grid['spectra_mu'] = [[0] * star_grid['n_grid'] for i in range(star_grid['n_grid'])]

    for x in range(0, star_grid['n_grid']):
        for y in range(0, star_grid['n_grid']):

            if star_grid['outside'][y, x]: continue

            index_closer = np.abs(
                synthesis['surface']['mu'] - star_grid['mu'][y, x]).argmin()  # take the index of the closer value

            if star_grid['mu'][y, x] in synthesis['surface']['mu']:
                star_grid['spectra_mu'][x][y] = synthesis['surface']['flux'][:, index_closer]
                continue
            elif index_closer == synthesis['surface']['n_mu'] - 1 or \
                    synthesis['surface']['mu'][index_closer] > star_grid['mu'][y, x]:
                mu_ind0 = index_closer - 1
                mu_ind1 = index_closer
            else:
                mu_ind0 = index_closer
                mu_ind1 = index_closer + 1

            diff_mu = synthesis['surface']['mu'][mu_ind1] - synthesis['surface']['mu'][mu_ind0]

            star_grid['spectra_mu'][x][y] = synthesis['surface']['flux'][:, mu_ind0] \
                                            + (star_grid['mu'][y, x] - synthesis['surface']['mu'][mu_ind0]) / diff_mu \
                                            * (synthesis['surface']['flux'][:, mu_ind1]
                                               - synthesis['surface']['flux'][:, mu_ind0])

    """ Computation of the continuum level (total flux is already normalized)"""
    star_grid['continuum'] = [[0] * star_grid['n_grid'] for i in range(star_grid['n_grid'])]

    spectra_window = ((synthesis['surface']['wave'] > clv_rm_dict['continuum_range'][0]) &
                      (synthesis['surface']['wave'] < clv_rm_dict['continuum_range'][1]))

    for x in range(0, star_grid['n_grid']):
        for y in range(0, star_grid['n_grid']):

            if star_grid['outside'][y, x]: continue
            star_grid['continuum'][x][y] = np.median(star_grid['spectra_mu'][x][y][spectra_window])

    star_grid['continuum_level'] = np.sum(star_grid['continuum'])

    for night in night_dict:
        """ Retrieving the list of observations"""

        print()
        print('compute_CLV_RM_modelling                   Night: ', night)
        try:
            clv_rm_modelling = load_from_cpickle('clv_rm_modelling', config_in['output'], night)
            continue
        except:
            print()
            print('  No CLV & RM correction files found, computing now ')

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        instrument = night_dict[night]['instrument']

        clv_rm_modelling = {
            'common': {
                'wave': synthesis['surface']['wave_out'],
                'step': synthesis['surface']['step_out'],
                'norm': synthesis['total']['norm_out'],
                'continuum_level': star_grid['continuum_level']
            }
        }

        clv_rm_modelling['common']['convolution_dlambda'] = \
            np.median(clv_rm_modelling['common']['wave']) / instrument_dict[instrument]['resolution']
        clv_rm_modelling['common']['convolution_sigma'] = \
            clv_rm_modelling['common']['convolution_dlambda'] / np.median(clv_rm_modelling['common']['step'])
        gaussian = Gaussian1DKernel(stddev=clv_rm_modelling['common']['convolution_sigma'])

        clv_rm_modelling['common']['norm_convolved'] = convolve(clv_rm_modelling['common']['norm'], gaussian)

        processed = {}

        print()
        for obs in lists['observations']:

            print('  Computing CLV+RM correction for ', obs)

            processed[obs] = {}
            clv_rm_modelling[obs] = {}

            n_oversampling = int(observational_pams[obs]['EXPTIME'] / clv_rm_dict['time_step'])

            if n_oversampling % 2 == 0:  n_oversampling += 1
            half_time = observational_pams[obs]['EXPTIME'] / 2 / 86400.

            processed[obs]['bjd_oversampling'] = np.linspace(observational_pams[obs]['BJD'] - half_time,
                                                             observational_pams[obs]['BJD'] + half_time,
                                                             n_oversampling, dtype=np.double)

            if planet_dict['orbit'] == 'circular':
                # Time of pericenter concides with transit time, if we assume e=0 and omega=np.pi/2.
                eccentricity = 0.00
                omega_rad = np.pi / 2.
                # Tcent is assumed as reference time
                Tref = planet_dict['reference_time_of_transit'][0]
                Tcent_Tref = 0.000
            else:
                omega_rad = planet_dict['omega'][0] * deg2rad
                Tref = planet_dict['reference_time']
                Tcent_Tref = planet_dict['reference_time_of_transit'][0] - Tref
                eccentricity = planet_dict['eccentricity'][0]
            inclination_rad = planet_dict['inclination'][0] * deg2rad

            true_anomaly, orbital_distance_ratio = kepler_true_anomaly_orbital_distance(
                processed[obs]['bjd_oversampling'] - Tref,
                Tcent_Tref,
                planet_dict['period'][0],
                eccentricity,
                omega_rad,
                planet_dict['semimajor_axis_ratio'][0])

            """ planet position during its orbital motion, in unit of stellar radius"""
            # Following Murray+Correia 2011 , with the argument of the ascending node set to zero.
            # 1) the ascending node coincide with the X axis
            # 2) the reference plance coincide with the plane of the sky
            processed[obs]['planet_position'] = {
                'xp': -orbital_distance_ratio * (np.cos(omega_rad + true_anomaly)),
                'yp': orbital_distance_ratio * (np.sin(omega_rad + true_anomaly) * np.cos(inclination_rad)),
                'zp': orbital_distance_ratio * (np.sin(inclination_rad) * np.sin(omega_rad + true_anomaly))
            }
            # projected distance of the planet's center to the stellar center
            processed[obs]['planet_position']['rp'] = np.sqrt(processed[obs]['planet_position']['xp'] ** 2 \
                                                              + processed[obs]['planet_position']['yp'] ** 2)

            # obscured flux integrated over the full epoch
            clv_rm_modelling[obs]['missing_flux'] = np.zeros(synthesis['surface']['size_out'], dtype=np.double)

            # iterating on the sub-exposures
            for j, zeta in enumerate(processed[obs]['planet_position']['zp']):

                if zeta > 0 and processed[obs]['planet_position']['rp'][j] < 1 + planet_dict['radius_ratio'][0]:
                    # the planet is in the foreground or inside the stellar disk, continue
                    # adjustment: computation is performed even if only part of the planet is shadowing the star

                    rd = np.sqrt((processed[obs]['planet_position']['xp'][j] - star_grid['xc']) ** 2 + \
                                 (processed[obs]['planet_position']['yp'][j] - star_grid['yc']) ** 2)

                    # iterating on the cell grid
                    for x in range(0, star_grid['n_grid']):
                        for y in range(0, star_grid['n_grid']):

                            # skip the step if the cell is outside the stellar disk
                            # or if the cell is not shadowed by the planet
                            if star_grid['outside'][y, x] or rd[y, x] > planet_dict['radius_ratio'][0]: continue

                            flux_tmp = rebin_1d_to_1d(synthesis['surface']['wave'],
                                                      synthesis['surface']['step'],
                                                      star_grid['spectra_mu'][x][y],
                                                      clv_rm_modelling['common']['wave'],
                                                      clv_rm_modelling['common']['step'],
                                                      rv_shift=star_grid['v_star'][y, x],
                                                      method='exact_flux',
                                                      preserve_flux=False)

                            # fixing zero values that may have been introduced by
                            # the rebinning process from an extremely irregular sampling

                            ind_sel = np.where(flux_tmp < 0.)[0]
                            for ii in ind_sel:
                                if ii == 0:
                                    flux_tmp[ii] = flux_tmp[ii + 1]
                                elif ii == np.size(flux_tmp) - 1:
                                    flux_tmp[ii] = flux_tmp[ii - 1]
                                else:
                                    flux_tmp[ii] = (flux_tmp[ii - 1] + flux_tmp[ii + 1]) / 2.

                            clv_rm_modelling[obs]['missing_flux'] += flux_tmp

            clv_rm_modelling[obs]['missing_flux'] /= n_oversampling
            clv_rm_modelling[obs]['stellar_spectra'] = clv_rm_modelling['common']['norm'] \
                                                       - (clv_rm_modelling[obs]['missing_flux']
                                                          / clv_rm_modelling['common']['continuum_level'])

            clv_rm_modelling[obs]['stellar_spectra_convolved'] = \
                convolve(clv_rm_modelling[obs]['stellar_spectra'], gaussian)

        save_to_cpickle('clv_rm_modelling', clv_rm_modelling, config_in['output'], night)

    if not config_in['settings'].get('full_output', False):
        del star_grid['spectra_mu']

    save_to_cpickle('clv_rm_star_grid', star_grid, config_in['output'])
    save_to_cpickle('clv_rm_synthesis', synthesis, config_in['output'])


def plot_clv_rm_modelling(config_in, night_input=''):
    night_dict = from_config_get_nights(config_in)

    synthesis = load_from_cpickle('clv_rm_synthesis', config_in['output'])
    star_grid = load_from_cpickle('clv_rm_star_grid', config_in['output'])

    if night_input == '':
        # Visualize the mu of star
        fig = plt.figure(figsize=(8, 6.5))

        plt.title('Limb angle')
        plt.contourf(star_grid['xx'], star_grid['xx'], star_grid['mu'], 60, cmap=plt.cm.viridis)
        plt.colorbar(label='$\mu$')  # draw colorbar
        # plot data points.
        plt.xlim(-1, 1)
        plt.ylim(-1, 1)
        plt.xlabel('x [R_s]')
        plt.ylabel('y [R_s]')
        plt.show()

        # Visualize the RV of star
        fig = plt.figure(figsize=(8, 6.5))

        # CS = plt.contour(xx,xx,v_star,50,linewidths=0.5,colors='k')
        plt.title('Radial velocity field')

        plt.contourf(star_grid['xx'], star_grid['xx'], star_grid['v_star'], 100, cmap=plt.cm.seismic)
        plt.colorbar(label='v_star')  # draw colorbar
        plt.xlim(-1, 1)
        plt.ylim(-1, 1)
        plt.xlabel('x [R_s]')
        plt.ylabel('y [R_s]')
        plt.show()

    if night_input == '':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)

    for night in night_list:

        lists = load_from_cpickle('lists', config_in['output'], night)
        clv_rm_modelling = load_from_cpickle('clv_rm_modelling', config_in['output'], night)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        colors_properties, colors_plot, colors_scatter = make_color_array_matplotlib3(lists, observational_pams)

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(2, 2, width_ratios=[50, 1])

        ax1 = plt.subplot(gs[0, 0])
        ax2 = plt.subplot(gs[1, 0], sharex=ax1, sharey=ax1)
        cbax1 = plt.subplot(gs[:, 1])

        for obs in lists['transit_in']:
            ax1.plot(clv_rm_modelling['common']['wave'],
                     clv_rm_modelling[obs]['stellar_spectra'],
                     color=colors_plot['mBJD'][obs], alpha=0.2)

            ax1.plot(clv_rm_modelling['common']['wave'],
                     clv_rm_modelling[obs]['missing_flux'] / clv_rm_modelling['common']['continuum_level'],
                     color=colors_plot['mBJD'][obs], alpha=0.2)

        for obs in lists['transit_out']:
            ax2.plot(clv_rm_modelling['common']['wave'],
                     clv_rm_modelling[obs]['stellar_spectra'],
                     color=colors_plot['mBJD'][obs], alpha=0.2)

        ax1.set_title('Night: {0:s} \n Input spectra'.format(night))
        ax2.set_title('Out of transit spectra')
        ax2.set_xlabel('$\lambda$ [$\AA$]')

        sm = plt.cm.ScalarMappable(cmap=colors_properties['cmap'], norm=colors_properties['norm']['mBJD'])
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()


"""from classes.kepler_exo import *

# Mass of the star HD189733 (in Solar masses)
#Ms = 0.823
Ms = 1.148
# Mass of the planet (in Solar masses)
#Mp = 1.138 / 1.047348644e3
Mp = 0.69 / 1.047348644e3

K1 = kepler_K1(Mp,Ms,3.52474854657,86.59,0.0082)
print K1


## update
"""

import matplotlib.pyplot as plt
import numpy as np
from SLOPpy.subroutines.constants import *
import argparse
from scipy.optimize import fsolve
from SLOPpy.subroutines.kepler_exo import *

def get_mass(M_star2, M_star1, Period, K1, e0):
    # M_star1, M_star2 in solar masses
    # P in days -> Period is converted in seconds in the routine
    # inclination assumed to be 90 degrees
    # Gravitational constant is given in m^3 kg^-1 s^-2
    # output in m/s
    output = K1 - (2. * np.pi * G_grav * M_sun / 86400.0) ** (1.0 / 3.0) * (1.000 / np.sqrt(1.0 - e0 ** 2.0)) * (
                                                                                                                    Period) ** (
                                                                                                                    -1.0 / 3.0) * (
                      M_star2 * (M_star1 + M_star2) ** (-2.0 / 3.0))
    return output




star_mass = 0.500
P = 5.
i = 90.
e = 0.00


planet_mass = np.arange(0,20, 0.1)
planet_K = planet_mass*0.

for i_val, m_val in enumerate(planet_mass):
    planet_K[i_val] = kepler_K1(m_val  * Mjups , star_mass, P, i, e)/ 1000.

plt.plot(planet_mass, planet_K)
plt.show()

#sampler = args.sample[0]
#file_conf = args.config_file[0]



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.shortcuts import *

from scipy.stats import linregress


__all__ = ["compute_second_telluric_correction_on_transmission", "plot_second_telluric_correction_on_transmission"]

subroutine_name = 'second_telluric_correction_on_transmission'


def compute_second_telluric_correction_on_transmission(config_in):
    night_dict = from_config_get_nights(config_in)
    shared_data = load_from_cpickle('shared', config_in['output'])

    for night in night_dict:

        try:
            transmission = load_from_cpickle('transmission_planetRF_second_correction', config_in['output'], night)
            continue
        except:
            print("No transmission spectra with second correction found, computing now ")
            print()

        print()
        print("compute_second_telluric_correction_on_transmission     Night: ", night)

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        transmission = load_from_cpickle('transmission_planetRF', config_in['output'], night)

        telluric = load_from_cpickle('telluric', config_in['output'], night)

        calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
        input_data = retrieve_observations(config_in['output'], night, lists['observations'], use_telluric=False)

        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        processed = {
            'subroutine': subroutine_name
        }

        for obs in lists['observations']:

            processed[obs] = {}


            processed[obs]['telluric_shifted'] = \
                rebin_2d_to_1d(input_data[obs]['wave'],
                               input_data[obs]['step'],
                               telluric[obs]['spectrum'],
                               telluric[obs]['spectrum'],
                               transmission['wave'],
                               transmission['step'],
                               rv_shift=observational_pams[obs]['rv_shift_ORF2PRF'],
                               preserve_flux=False,
                               skip_blaze_correction=True)


            processed[obs]['selection'] = (np.abs(1.0000-transmission[obs]['rescaled']) < 2*np.std(transmission[obs]['rescaled'])) \
                                          & (np.abs(1.0000-processed[obs]['telluric_shifted']) > 0.02)

            transmission[obs]['slope'], \
            transmission[obs]['intercept'], \
            transmission[obs]['rvalue'], \
            transmission[obs]['pvalue'], \
            transmission[obs]['stderr'], = linregress(processed[obs]['telluric_shifted'][processed[obs]['selection']],
                                                      transmission[obs]['rescaled'][processed[obs]['selection']])

            transmission[obs]['rescaled'] += 1.000 - (transmission[obs]['intercept'] +
                                                               transmission[obs]['slope']*processed[obs]['telluric_shifted'])

        array_average = np.zeros([len(lists['transit_in']), transmission['size']])
        weights_average = np.zeros([len(lists['transit_in']), transmission['size']])
        for i, obs in enumerate(lists['transit_in']):
            array_average[i, :] = transmission[obs]['rescaled'][:]
            weights_average[i, :] = 1./(transmission[obs]['rescaled_err']**2.)


        transmission['average'], transmission['sum_weights'] = np.average(
            array_average, axis=0, weights=weights_average, returned=True)
        transmission['average_err'] = 1./np.sqrt(transmission['sum_weights'])

        array_average = np.zeros([len(lists['transit_out']), transmission['size']])
        weights_average = np.zeros([len(lists['transit_out']), transmission['size']])

        for i, obs in enumerate(lists['transit_out']):
            array_average[i, :] = transmission[obs]['rescaled'][:]
            weights_average[i, :] = 1./(transmission[obs]['rescaled_err']**2.)


        transmission['average_out'], transmission['sum_weights_out'] = np.average(
            array_average, axis=0, weights=weights_average, returned=True)
        transmission['average_out_err'] = 1./np.sqrt(transmission['sum_weights_out'])

        save_to_cpickle('transmission_planetRF_second_correction_processed', processed, config_in['output'], night)
        save_to_cpickle('transmission_planetRF_second_correction', transmission, config_in['output'], night)


def plot_second_telluric_correction_on_transmission(config_in, night_input=''):
    night_dict = from_config_get_nights(config_in)

    if night_input == '':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)

    for night in night_list:

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)

        previous_transmission = load_from_cpickle('transmission_planetRF', config_in['output'], night)

        transmission = load_from_cpickle('transmission_planetRF_second_correction', config_in['output'], night)
        processed = load_from_cpickle('transmission_planetRF_second_correction_processed', config_in['output'], night)

        f, (ax1, ax2) = plt.subplots(2, sharex=True, sharey=True)

        cmap = plt.cm.Spectral
        line_colors = cmap(np.linspace(0, 1, len(lists['observations'])))

        for i, obs in enumerate(lists['observations']):

            ax1.axhline(1.+i/10., c='k', zorder=0)
            ax2.axhline(1.+i/10., c='k', zorder=0)

            ax1.scatter(processed[obs]['telluric_shifted'], previous_transmission[obs]['rescaled']+i / 10., s=2,
                        c=np.atleast_2d(line_colors[i]), zorder=1)

            ax2.scatter(processed[obs]['telluric_shifted'], transmission[obs]['rescaled'] + i / 10., s=2,
                        #c=np.atleast_2d(line_colors[i]), zorder=2)
                        c = 'r', zorder = 2)

        ax1.set_xlim(0.80, 1.02)
        plt.show()



        """ Creation of the color array, based on the BJD of the observations
        """
        bjd = []
        am = []

        for obs in lists['observations']:
            bjd.append(transmission[obs]['BJD'] - 2450000.0)
            am.append(transmission[obs]['AIRMASS'])

        colors = np.asarray(bjd)
        cmap = plt.cm.Spectral
        line_colors = cmap(np.linspace(0, 1, len(lists['observations'])))

        fig = plt.figure(figsize=(12, 6))
        gs = GridSpec(1, 2, width_ratios=[50, 1])
        ax = plt.subplot(gs[0, 0])
        cbax1 = plt.subplot(gs[:, 1])

        i_shift = 0.00
        for i, obs in enumerate(lists['observations']):
            ax.errorbar(transmission['wave'],
                        transmission[obs]['rescaled']-i_shift,
                        yerr=transmission[obs]['rescaled_err'],
                        marker = 'o', c=line_colors[i], ms=1, alpha=0.5)

            i_shift += 0.05

        ax.set_ylim(0.0-i_shift, 1.2)
        ax.set_xlabel('$\lambda$ [$\AA$]')
        ax.legend(loc=3)
        ax.set_title('Night: ' + night)

        sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=colors[0], vmax=colors[-1]))
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()




from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.shortcuts import *
from SLOPpy.subroutines.plot_subroutines import *
from SLOPpy.pca_preparation import compute_pca_preparation

from PyAstronomy import pyasl

from scipy.interpolate import UnivariateSpline


__all__ = ['compute_sysrem_correction',
           'plot_sysrem_correction']

def compute_sysrem_correction(config_in):

    subroutine_name = 'sysrem_correction'
    compute_pca_preparation(config_in)
    print()

    night_dict = from_config_get_nights(config_in)
    pca_parameters = from_config_get_pca_parameters(config_in)

    for night in night_dict:

        try:
            sysrem_output = load_from_cpickle('transmission_preparation',
                                          config_in['output'],
                                          night)
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Retrieved'))
            continue
        except:
            print("{0:45s} Night:{1:15s}   {2:s}".format(subroutine_name, night, 'Computing'))
            print()


        n_iter = pca_parameters.get('iterations',5 )
        ref_iter = pca_parameters.get('ref_iteration',3 )
        sysrem_output = {
            'subroutine': subroutine_name,
            'iterations': n_iter,
            'pca_output': True,
            'ref_iteration': ref_iter
        }


        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        preparation = load_from_cpickle('pca_preparation', config_in['output'], night)

        n_obs, n_orders, n_pixels = np.shape(preparation['stack_e2ds']) 

        model_iter = np.ones([n_iter, n_obs, n_orders, n_pixels], dtype=np.double)
        model_out = np.ones([n_iter, n_obs, n_orders, n_pixels], dtype=np.double)

        for order in range(0, observational_pams['n_orders']):
            obs = preparation['stack_e2ds'][:,order,:]
            sigs = preparation['stack_e2ds_err'][:,order,:]

            sr = pyasl.SysRem(obs, sigs)
            previous_residuals = obs.copy()

            for it in range(0, n_iter):
                r, a, c = sr.iterate()
                model_iter[it,:,order,:] =  previous_residuals - r
                previous_residuals = r.copy()


        for it in range(0, n_iter):

            # Full model is the sum of all the models until the given iteration 
            model_out[it,:, :, :] = np.sum(model_iter[:it+1,:, :, :], axis=0)

            #import matplotlib.pyplot as plt
            #plt.figure()
            #plt.title("Model " + repr(it))
            #plt.imshow( model_out[it,:, 10, :], origin='lower', aspect="auto")
            #plt.show()

            it_string = str(it).zfill(2)

            sysrem_output[it_string] = {
                'model': model_out[it,:, :, :]
            }

            for i_obs, obs in enumerate(lists['observations']):

                sysrem_output[it_string][obs] = {}
                sysrem_output[it_string][obs]['ratio']  = preparation['stack_e2ds'][i_obs, :, :]/model_out[it, i_obs, :, :]
                sysrem_output[it_string][obs]['ratio_err']  = preparation['stack_e2ds_err'][i_obs, :, :]/model_out[it, i_obs, :, :]


        save_to_cpickle('transmission_preparation', sysrem_output, config_in['output'], night)

    print()
    """ Keep going from here after preparation, unless the subroutines has been called just
        to preform the data preparation step
    """


def plot_sysrem_correction(config_in, night_input=''):

    subroutine_name = 'transmission_spectrum_preparation'

    night_dict = from_config_get_nights(config_in)

    if night_input == '':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)




    for night in night_list:

        """ Retrieving the list of observations"""
        lists = load_from_cpickle('lists', config_in['output'], night)
        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

        # ! To be removed when testing is done
        # ! This plots do not make any sense anymore 
        input_data = retrieve_observations(config_in['output'], night, lists['observations'])


        """ Retrieving the analysis"""
        try:
            preparation = load_from_cpickle('transmission_preparation', config_in['output'], night)
        except:
            print("No transmission spectrum results, no plots")
            print()
            continue


        #from SLOPpy.subroutines.lines_fit_functions import logprob_case12
        from matplotlib.colors import BoundaryNorm
        from matplotlib.ticker import MaxNLocator

        len_y = len(lists['observations'])
        len_x = 4096
        order= 11

        time_from_transit = np.empty(len_y, dtype=np.double)
        plot_data = np.empty([len_y, len_x], dtype=np.double)

        for i_obs, obs in enumerate(lists['observations']):
            print(np.median(preparation[obs]['deblazed'][order ,:]))
            time_from_transit[i_obs] =  input_data[obs]['BJD'] - observational_pams['time_of_transit']
            plot_data[i_obs, :] = preparation[obs]['deblazed'][order ,:]/ np.median(preparation[obs]['ratio'][order ,:])
            wave = preparation[obs]['wave'][order, :]

        wave_meshgrid, time_meshgrid = np.meshgrid(wave, time_from_transit)

        print('COOLWARM')
        cmap = plt.get_cmap('coolwarm')

        levels = MaxNLocator(nbins=15).tick_values(
            plot_data.min(), plot_data.max())
        norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)
        plt.figure(figsize=(15, 10))

        PCF = plt.contourf(wave_meshgrid, time_meshgrid,
                            plot_data, levels=levels, cmap=cmap)
        cbar = plt.colorbar(PCF)
        cbar.ax.set_ylabel('Intensity')
        plt.show()

        res = plot_data * 1.
        from scipy.interpolate import UnivariateSpline
        for ii in range(0,4096):
            spl = UnivariateSpline(time_from_transit, plot_data[:, ii])
            val = spl(time_from_transit)
            res[:,ii] -= val
            res[:,ii] /= val



        print('COOLWARM')
        cmap = plt.get_cmap('coolwarm')

        levels = MaxNLocator(nbins=10).tick_values(
            -0.05, 0.05)
        norm = BoundaryNorm(levels, ncolors=cmap.N, clip=True)
        plt.figure(figsize=(15, 10))

        PCF = plt.contourf(wave_meshgrid, time_meshgrid,
                            res, levels=levels, cmap=cmap)
        cbar = plt.colorbar(PCF)
        cbar.ax.set_ylabel('Intensity')
        plt.show()





        """ Creation of the color array, based on the BJD of the observations
        """
        colors_properties, colors_plot, colors_scatter = make_color_array_matplotlib3(lists, observational_pams)

        fig = plt.figure(figsize=(12, 6))

        gs = GridSpec(1, 2, width_ratios=[50, 1])
        ax1 = plt.subplot(gs[0, 0])
        #ax2 = plt.subplot(gs[1, 0], sharex=ax1, sharey=ax1)
        cbax1 = plt.subplot(gs[:, 1])

        for obs in lists['transit_in']:

            preparation[obs]['rescaling'], \
            preparation[obs]['rescaled'], \
            preparation[obs]['rescaled_err'] = perform_rescaling(
                preparation[obs]['wave'],
                preparation[obs]['deblazed'] / (input_data[obs]['step'] / np.median(input_data[obs]['step'])),
                preparation[obs]['deblazed_err'] / (input_data[obs]['step'] / np.median(input_data[obs]['step'])),
                observational_pams['wavelength_rescaling'])

            ax1.scatter(preparation[obs]['wave'],
                    preparation[obs]['rescaled'],
                    s=1, alpha=0.25,
                    color=colors_plot['mBJD'][obs])

        sm = plt.cm.ScalarMappable(cmap=colors_properties['cmap'], norm=colors_properties['norm']['mBJD'])
        sm.set_array([])  # You have to set a dummy-array for this to work...
        cbar = plt.colorbar(sm, cax=cbax1)
        cbar.set_label('BJD - 2450000.0')
        fig.subplots_adjust(wspace=0.05, hspace=0.4)
        plt.show()



from __future__ import print_function, division
from SLOPpy.subroutines.common import *
from SLOPpy.subroutines.spectral_subroutines import *
from SLOPpy.subroutines.io_subroutines import *
from SLOPpy.subroutines.fit_subroutines import *
from SLOPpy.subroutines.shortcuts import *
from SLOPpy.subroutines.rebin_subroutines import *
from astropy.convolution import convolve, Box1DKernel

__all__ = ['compute_transmission_lightcurve_planetRF',
           'plot_transmission_lightcurve_planetRF',
           'compute_transmission_lightcurve_stellarRF',
           'plot_transmission_lightcurve_stellarRF',
           'compute_transmission_lightcurve_observerRF',
           'plot_transmission_lightcurve_observerRF',
           'compute_transmission_lightcurve',
           'plot_transmission_lightcurve'
           ]


def compute_transmission_lightcurve_planetRF(config_in, lines_label):
    compute_transmission_lightcurve(config_in, lines_label, reference='planetRF')


def plot_transmission_lightcurve_planetRF(config_in, night_input):
    plot_transmission_lightcurve(config_in, night_input, reference='planetRF')


def compute_transmission_lightcurve_stellarRF(config_in, lines_label):
    compute_transmission_lightcurve(config_in, lines_label, reference='stellarRF')


def plot_transmission_lightcurve_stellarRF(config_in, night_input):
    plot_transmission_lightcurve(config_in, night_input, reference='stellarRF')


def compute_transmission_lightcurve_observerRF(config_in, lines_label):
    compute_transmission_lightcurve(config_in, lines_label, reference='observerRF')


def plot_transmission_lightcurve_observerRF(config_in, night_input):
    plot_transmission_lightcurve(config_in, night_input, reference='observerRF')

subroutine_name = 'transmission_lightcurve'
pick_files =  'transmission_spectrum'

def compute_transmission_lightcurve(config_in, lines_label, reference='planetRF'):

    do_average_instead_of_sum = True

    night_dict = from_config_get_nights(config_in)
    #instrument_dict = from_config_get_instrument(config_in)
    #system_dict = from_config_get_system(config_in)
    planet_dict = from_config_get_planet(config_in)

    spectral_lines = from_config_get_spectral_lines(config_in)
    #shared_data = load_from_cpickle('shared', config_in['output'])

    lines_dict = spectral_lines[lines_label] # from_config_get_transmission_lightcurve(config_in)

    if 'full_transit_duration' in planet_dict:
        full_transit_duration = planet_dict['total_transit_duration'][0]
    else:
        full_transit_duration = planet_dict['transit_duration'][0]

    if 'total_transit_duration' in planet_dict:
        total_transit_duration = planet_dict['total_transit_duration'][0]
    else:
        total_transit_duration = planet_dict['transit_duration'][0]

    transit_in_bins = np.linspace(
        -total_transit_duration/2./planet_dict['period'][0],
        total_transit_duration/2./planet_dict['period'][0],
        6
    )
    transit_full_bins = np.linspace(
        -full_transit_duration/2./planet_dict['period'][0],
        full_transit_duration/2./planet_dict['period'][0],
        6
    )

    transit_in_step = np.average(transit_in_bins[1:]-transit_in_bins[:-1])
    transit_full_step = np.average(transit_full_bins[1:]-transit_full_bins[:-1])

    output_list = ['user',
                    'mcmc_night_MED',
                    'mcmc_night_MAP',
                    'mcmc_global_MED',
                    'mcmc_global_MAP']

    append_list = ['', '_uncorrected', '_clv_model']

    for output_selection in output_list:
        skip_iteration = False

        for night in night_dict:

            print("compute_transmission_lightcurve            Night: ", night)

            try:
                transmission = load_from_cpickle(pick_files+'_'+reference + '_' + output_selection, config_in['output'], night, lines_label)
            except (FileNotFoundError, IOError):
                print('No transmission spectra found for case:{0:s}, be sure to run transmission_spectra before this step'.format(output_selection))
                skip_iteration = True

            if skip_iteration: continue

            try:
                lightcurve = load_from_cpickle(subroutine_name +'_'+reference+ '_' + output_selection, config_in['output'], night, lines_label)
                print()
                continue
            except (FileNotFoundError, IOError):
                print("  No transmission_lightcurve file found, computing now ")
                print()


            """ Retrieving the list of observations"""
            lists = load_from_cpickle('lists', config_in['output'], night)

            #calib_data = load_from_cpickle('calibration_fibA', config_in['output'], night)
            #input_data = retrieve_observations( config_in['output'], night, lists['observations'])
            observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)

            # doublet sodium in the lab reference frame

            """
            C stands for central
            """
            C_bands = {}
            for passband_key, passband_val in lines_dict['passbands'].items():
                C_bands[passband_key] = {}
                for line_key, line_val in lines_dict['lines'].items():
                    C_bands[passband_key][line_key] = (np.abs(transmission['wave'] - line_val) * 2. < passband_val)

            """
            S stands for side
            """
            S_bands = {}
            for band_key, band_val in lines_dict['continuum'].items():
                S_bands[band_key] = (transmission['wave'] >= band_val[0]) & (transmission['wave'] <= band_val[1])


            processed = {
                'subroutine': subroutine_name
            }

            lightcurve = {
                'subroutine': subroutine_name,
                'arrays': {
                    'observations': {
                        'obs_name': np.zeros(len(lists['observations']), dtype=str),
                        'phase': np.zeros(len(lists['observations'])),
                    },
                    'transit_in': {},
                    'transit_full': {},
                    'transit_out': {},
                },
                'C_bands': C_bands,
                'S_bands': S_bands,
                'bins': {
                    'transit_in_bins': transit_in_bins,
                    'transit_in_step': transit_in_step,
                    'transit_full_bins': transit_full_bins,
                    'transit_full_step': transit_full_step
                }
            }
            """ Adding the C-bands arrays to the dictionary"""

            for band_key in C_bands:
                for name_append in append_list:
                    lightcurve['arrays']['observations']['delta_' + band_key + name_append] = np.zeros([len(lists['observations']), 2])

            transit_out_flag = np.zeros(len(lists['observations']), dtype=bool)
            transit_in_flag = np.zeros(len(lists['observations']), dtype=bool)
            transit_full_flag = np.zeros(len(lists['observations']), dtype=bool)

            for n_obs, obs in enumerate( lists['observations']):

                processed[obs] = {}
                lightcurve[obs] = {}


                try:
                    phase_internal = (observational_pams[obs]['BJD'] - night_dict[night]['time_of_transit'][0])/planet_dict['period'][0]
                except:
                    phase_internal = (observational_pams[obs]['BJD'] - night_dict[night]['time_of_transit'])/planet_dict['period'][0]


                processed[obs]['bands'] = {
                    'phase': phase_internal
                }
                processed[obs]['bands_uncorrected'] = {
                    'phase': phase_internal
                }
                processed[obs]['bands_clv_model'] = {
                    'phase': phase_internal
                }

                processed[obs]['s_integrated'] = 0.000
                processed[obs]['s_integrated_uncorrected'] = 0.000
                processed[obs]['s_integrated_clv_model'] = 0.000
                processed[obs]['s_sigmaq_sum'] = 0.000

                n_bands = 0.0
                for band_key, band_val in S_bands.items():
                    if do_average_instead_of_sum:
                        processed[obs]['bands'][band_key] =  \
                            [np.average(transmission[obs]['normalized'][band_val]),
                            np.sum((transmission[obs]['normalized_err'][band_val])**2)
                            /len(transmission[obs]['normalized_err'][band_val])**2]
                        processed[obs]['bands_uncorrected'][band_key] =  \
                            [np.average(transmission[obs]['normalized_uncorrected'][band_val]),
                            np.sum((transmission[obs]['normalized_uncorrected_err'][band_val])**2)
                            /len(transmission[obs]['normalized_uncorrected_err'][band_val])**2]
                        processed[obs]['bands_clv_model'][band_key] =  \
                            [np.average(transmission[obs]['clv_model_rebinned'][band_val]),
                            np.sum((transmission[obs]['normalized_err'][band_val])**2)
                            /len(transmission[obs]['normalized_err'][band_val])**2]
                    else:
                        processed[obs]['bands'][band_key] =  \
                            [np.sum(transmission[obs]['normalized'][band_val]),
                            np.sum((transmission[obs]['normalized_err'][band_val])**2)]
                        processed[obs]['bands_uncorrected'][band_key] =  \
                            [np.sum(transmission[obs]['normalized_uncorrected'][band_val]),
                            np.sum((transmission[obs]['normalized_uncorrected_err'][band_val])**2)]
                        processed[obs]['bands_clv_model'][band_key] =  \
                            [np.sum(transmission[obs]['clv_model_rebinned'][band_val]),
                            np.sum((transmission[obs]['normalized_err'][band_val])**2)]
                    processed[obs]['s_integrated'] += processed[obs]['bands'][band_key][0]
                    processed[obs]['s_integrated_uncorrected'] += processed[obs]['bands_uncorrected'][band_key][0]
                    processed[obs]['s_integrated_clv_model'] += processed[obs]['bands_clv_model'][band_key][0]
                    processed[obs]['s_sigmaq_sum'] += processed[obs]['bands'][band_key][1]
                    n_bands += 1.

                processed[obs]['s_integrated'] /= n_bands
                processed[obs]['s_integrated_uncorrected'] /= n_bands
                processed[obs]['s_integrated_clv_model'] /= n_bands
                processed[obs]['s_sigmaq_sum'] /= n_bands**2

                #processed[obs]['s_factor'] =np.power(s_integrated, -2.0)
                #processed[obs]['s_factor_clv_model'] =np.power(s_integrated, -2.0)
                #processed[obs]['s_factor_uncorrected'] = np.power(s_integrated, -2.0)

                for band_key, band_dict in C_bands.items():
                    processed[obs]['bands'][band_key] = {}
                    processed[obs]['bands_uncorrected'][band_key] = {}
                    processed[obs]['bands_clv_model'][band_key] = {}

                    processed[obs]['c_integrated'] = 0.000
                    processed[obs]['c_integrated_uncorrected'] = 0.000
                    processed[obs]['c_integrated_clv_model'] = 0.000
                    processed[obs]['c_sigmaq_sum'] = 0.000

                    n_bands = 0.0
                    for line_key, line_val in band_dict.items():
                        if do_average_instead_of_sum:
                            processed[obs]['bands'][band_key][line_key] = \
                                [np.average(transmission[obs]['normalized'][line_val]),
                                np.sum((transmission[obs]['normalized_err'][line_val])**2)
                                / len(transmission[obs]['normalized_err'][line_val])**2]
                            processed[obs]['bands_uncorrected'][band_key][line_key] = \
                                [np.average(transmission[obs]['normalized_uncorrected'][line_val]),
                                np.sum((transmission[obs]['normalized_uncorrected_err'][line_val])**2)
                                / len(transmission[obs]['normalized_uncorrected_err'][line_val])**2]
                            processed[obs]['bands_clv_model'][band_key][line_key] = \
                                [np.average(transmission[obs]['clv_model_rebinned'][line_val]),
                                np.sum((transmission[obs]['normalized_err'][line_val])**2)
                                / len(transmission[obs]['normalized_err'][line_val])**2]
                        else:
                            processed[obs]['bands'][band_key][line_key] = \
                                [np.sum(transmission[obs]['normalized'][line_val]),
                                np.sum((transmission[obs]['normalized_err'][line_val]) ** 2)]
                            processed[obs]['bands_uncorrected'][band_key][line_key] = \
                                [np.sum(transmission[obs]['normalized_uncorrected'][line_val]),
                                np.sum((transmission[obs]['normalized_uncorrected_err'][line_val]) ** 2)]
                            processed[obs]['bands_clv_model'][band_key][line_key] = \
                                [np.sum(transmission[obs]['clv_model_rebinned'][line_val]),
                                np.sum((transmission[obs]['normalized_err'][line_val]) ** 2)]

                        processed[obs]['c_integrated'] += processed[obs]['bands'][band_key][line_key][0]
                        processed[obs]['c_integrated_uncorrected'] += processed[obs]['bands_uncorrected'][band_key][line_key][0]
                        processed[obs]['c_integrated_clv_model'] += processed[obs]['bands_clv_model'][band_key][line_key][0]
                        processed[obs]['c_sigmaq_sum'] += processed[obs]['bands'][band_key][line_key][1]
                        n_bands += 1.

                    processed[obs]['c_integrated'] /= n_bands
                    processed[obs]['c_integrated_uncorrected'] /= n_bands
                    processed[obs]['c_integrated_clv_model'] /= n_bands
                    processed[obs]['c_sigmaq_sum'] /= n_bands ** 2

                    for name_append in append_list:

                        lightcurve[obs]['delta_' + band_key + name_append] = [processed[obs]['c_integrated' + name_append] 
                                                                            - processed[obs]['s_integrated' + name_append],
                                                                np.sqrt(processed[obs]['s_sigmaq_sum'] + processed[obs]['c_sigmaq_sum'])]

                        lightcurve['arrays']['observations']['delta_' + band_key + name_append][n_obs, :] = \
                            lightcurve[obs]['delta_' + band_key + name_append][:]

                lightcurve[obs]['phase'] = processed[obs]['bands']['phase']
                lightcurve['arrays']['observations']['obs_name'][n_obs] = obs
                lightcurve['arrays']['observations']['phase'][n_obs] = lightcurve[obs]['phase']

                if obs in lists['transit_out']:
                    transit_out_flag[n_obs] = True
                if obs in lists['transit_in']:
                    transit_in_flag[n_obs] = True
                if obs in lists['transit_full']:
                    transit_full_flag[n_obs] = True

            for band_key in C_bands:
                for name_append in append_list:
                    lightcurve['arrays']['rescaling_' + band_key + name_append] = \
                        np.average(lightcurve['arrays']['observations']['delta_' + band_key + name_append][transit_out_flag, 0], axis=0)

            sorting_index = np.argsort(lightcurve['arrays']['observations']['phase'])

            transit_out_flag = transit_out_flag[sorting_index]
            transit_in_flag = transit_in_flag[sorting_index]
            transit_full_flag = transit_full_flag[sorting_index]

            lightcurve['arrays']['observations']['obs_name'] = lightcurve['arrays']['observations']['obs_name'][sorting_index]
            lightcurve['arrays']['observations']['phase'] = lightcurve['arrays']['observations']['phase'][sorting_index]

            lightcurve['arrays']['transit_in']['obs_name'] = lightcurve['arrays']['observations']['obs_name'][transit_in_flag]
            lightcurve['arrays']['transit_in']['phase'] = lightcurve['arrays']['observations']['phase'][transit_in_flag]

            lightcurve['arrays']['transit_full']['obs_name'] = lightcurve['arrays']['observations']['obs_name'][transit_full_flag]
            lightcurve['arrays']['transit_full']['phase'] = lightcurve['arrays']['observations']['phase'][transit_full_flag]

            lightcurve['arrays']['transit_out']['obs_name'] = lightcurve['arrays']['observations']['obs_name'][transit_out_flag]
            lightcurve['arrays']['transit_out']['phase'] = lightcurve['arrays']['observations']['phase'][transit_out_flag]

            for band_key in C_bands:
                for name_append in append_list:

                    lightcurve['arrays']['observations']['delta_' + band_key + name_append] = \
                        lightcurve['arrays']['observations']['delta_' + band_key + name_append][sorting_index]
                        # / lightcurve['arrays']['rescaling_' + band_key]
                    lightcurve['arrays']['transit_in']['delta_' + band_key + name_append] = \
                        lightcurve['arrays']['observations']['delta_' + band_key + name_append][transit_in_flag]
                    lightcurve['arrays']['transit_full']['delta_' + band_key + name_append] = \
                        lightcurve['arrays']['observations']['delta_' + band_key + name_append][transit_full_flag]
                    lightcurve['arrays']['transit_out']['delta_' + band_key + name_append] = \
                        lightcurve['arrays']['observations']['delta_' + band_key + name_append][transit_out_flag]

            lightcurve['arrays']['observations']['transit_out_flag'] = transit_out_flag
            lightcurve['arrays']['observations']['transit_in_flag'] = transit_in_flag
            lightcurve['arrays']['observations']['transit_full_flag'] = transit_full_flag

            pre_duration = transit_full_bins[0] - lightcurve['arrays']['transit_out']['phase'][0]
            if pre_duration > 0:
                nsteps_pre = int(pre_duration/transit_full_step)
                if pre_duration % transit_full_step > 0.0:
                    nsteps_pre += 1
            else:
                nsteps_pre = 0

            post_duration = lightcurve['arrays']['transit_out']['phase'][-1] - transit_full_bins[-1]
            if post_duration > 0:
                nsteps_post = int(post_duration / transit_full_step)
                if post_duration % transit_full_step > 0.0:
                    nsteps_post += 1
            else:
                nsteps_post = 0

            transit_bins = np.arange(transit_full_bins[0]-nsteps_pre*transit_full_step,
                                    transit_full_bins[-1] + (nsteps_post+1.1) * transit_full_step,
                                    transit_full_step)

            lightcurve['binned'] = {
                'observations': {
                    'phase': np.zeros(len(transit_bins)),
                },
                'transit_in': {},
                'transit_full': {},
                'transit_out': {},
            }
            for band_key in C_bands:
                for name_append in append_list:
                    lightcurve['binned']['observations']['delta_' + band_key + name_append] = np.zeros([len(transit_bins), 2])

            transit_out_flag = np.zeros(len(transit_bins), dtype=bool)
            transit_in_flag = np.zeros(len(transit_bins), dtype=bool)
            transit_full_flag = np.zeros(len(transit_bins), dtype=bool)

            n_a = 0
            for nb in range(0, len(transit_bins)-1):
                sel = (lightcurve['arrays']['observations']['phase'] >= transit_bins[nb]) \
                    & (lightcurve['arrays']['observations']['phase'] < transit_bins[nb+1])

                if np.sum(sel) <= 0: continue
                lightcurve['binned']['observations']['phase'][n_a] = np.average(lightcurve['arrays']['observations']['phase'][sel])

                for band_key in C_bands:
                    for name_append in append_list:
                        lightcurve['binned']['observations']['delta_' + band_key + name_append][n_a, 0], sum_weights = np.average(
                            lightcurve['arrays']['observations']['delta_' + band_key + name_append][sel, 0],
                            weights=1. / lightcurve['arrays']['observations']['delta_' + band_key + name_append][sel, 1]**2,
                            returned=True)

                        lightcurve['binned']['observations']['delta_' + band_key + name_append][n_a, 1] = np.sqrt(1. / sum_weights)

                if np.abs(lightcurve['binned']['observations']['phase'][n_a]) >= \
                    total_transit_duration/2./planet_dict['period'][0]:
                        transit_out_flag[n_a] = True
                elif np.abs(lightcurve['binned']['observations']['phase'][n_a]) >= \
                    full_transit_duration/2./planet_dict['period'][0]:
                        transit_in_flag[n_a] = True
                else:
                    transit_full_flag[n_a] = True

                n_a += 1 # bins actually computed

            lightcurve['binned']['transit_in']['phase'] = lightcurve['binned']['observations']['phase'][transit_in_flag]
            lightcurve['binned']['transit_full']['phase'] = lightcurve['binned']['observations']['phase'][transit_full_flag]
            lightcurve['binned']['transit_out']['phase'] = lightcurve['binned']['observations']['phase'][transit_out_flag]
            lightcurve['binned']['observations']['phase'] = lightcurve['binned']['observations']['phase'][:n_a]

            for band_key in C_bands:
                for name_append in append_list:
                    lightcurve['binned']['transit_in']['delta_' + band_key + name_append] = \
                        lightcurve['binned']['observations']['delta_' + band_key + name_append][transit_in_flag, :]
                    lightcurve['binned']['transit_full']['delta_' + band_key + name_append] = \
                        lightcurve['binned']['observations']['delta_' + band_key + name_append][transit_full_flag, :]
                    lightcurve['binned']['transit_out']['delta_' + band_key + name_append] = \
                        lightcurve['binned']['observations']['delta_' + band_key + name_append][transit_out_flag, :]
                    lightcurve['binned']['observations']['delta_' + band_key + name_append] = \
                        lightcurve['binned']['observations']['delta_' + band_key + name_append][:n_a, :]

            save_to_cpickle(subroutine_name + '_'+reference + '_' + output_selection +'_processed', processed, config_in['output'], night, lines_label)
            save_to_cpickle(subroutine_name + '_'+reference + '_' + output_selection, lightcurve, config_in['output'], night, lines_label)


def plot_transmission_lightcurve(config_in, night_input='', reference='planetRF'):
    import matplotlib.pyplot as plt

    night_dict = from_config_get_nights(config_in)

    if night_input=='':
        night_list = night_dict
    else:
        night_list = np.atleast_1d(night_input)

    for night in night_list:

        print("plot_transmission_lightcurve               Night: ", night)

        """ Retrieving the analysis"""
        try:
            lightcurve = load_from_cpickle(subroutine_name+'_'+reference, config_in['output'], night)
        except:
            print()
            print("No transmission lightcurve dataset, no plots")
            continue

        observational_pams = load_from_cpickle('observational_pams', config_in['output'], night)
        C_bands = lightcurve['C_bands']

        for band_key in C_bands:
            plt.figure(figsize=(12, 6))
            plt.title('Transmission lightcurve - night {0:s} \n {1:s}'.format(night, band_key))
            plt.errorbar(lightcurve['arrays']['observations']['phase'],
                        lightcurve['arrays']['observations']['delta_' + band_key][:,0],
                         yerr= lightcurve['arrays']['observations']['delta_' + band_key][:,1],
                         fmt='.', c='k', alpha=0.25, label='observations')
            plt.errorbar(lightcurve['binned']['observations']['phase'],
                        lightcurve['binned']['observations']['delta_' + band_key][:, 0],
                         yerr= lightcurve['binned']['observations']['delta_' + band_key][:,1],
                         fmt='.', c='k', alpha=1.0, label='observations')

            plt.axvspan(-1, lightcurve['bins']['transit_in_bins'][0], alpha=0.25, color='green')
            plt.axvspan(lightcurve['bins']['transit_in_bins'][-1], 1., alpha=0.25, color='green')

            plt.axhline(0, c='C1')
            plt.xlim(lightcurve['arrays']['observations']['phase'][0]-0.01,
                     lightcurve['arrays']['observations']['phase'][-1]+0.01)
            plt.xlabel('$\lambda$ [$\AA$]')
            plt.ylabel('$\mathcal{R}$ - 1.')
            plt.legend()
            plt.show()







import numpy as np
from astropy.io import fits
from SLOPpy.subroutines.rebin_subroutines import *
from SLOPpy.subroutines.constants import *
from SLOPpy.subroutines.common import *

def DRSv3_map_orders_AB(properties, order_selection):

    n_orders_A = properties['n_orders_A']
    map_orders_A_full = np.arange(0, n_orders_A, dtype=np.int16)
    map_orders_BtoA = np.asarray(properties['orders_BtoA'])

    map_orders_A = []
    map_orders_B = []

    #if order_selection == 'red_ccd':
    #    working_A_full = map_orders_A_full[properties['red_ccd']]
    #    working_BtoA = map_orders_BtoA[properties['red_ccd']]
    if order_selection in ['red_ccd', 'blue_ccd', 'full_ccd']:
        working_A_full = map_orders_A_full[properties[order_selection]]
        working_BtoA = map_orders_BtoA[properties[order_selection]]

    elif order_selection:
        working_A_full = map_orders_A_full[order_selection]
        working_BtoA = map_orders_BtoA[order_selection]
    else:
        working_A_full = map_orders_A_full
        working_BtoA = map_orders_BtoA

    for order_A, order_B in zip(working_A_full, working_BtoA):
        if order_B < 0: continue
        map_orders_A.extend([order_A])
        map_orders_B.extend([order_B])

    return map_orders_A, map_orders_B


def DRSv3_give_back_selected_orders(properties, fiber, order_selection):

    map_orders_A, map_orders_B = DRSv3_map_orders_AB(properties, order_selection)
    if fiber != 'A':
        return map_orders_B
    else:
        return map_orders_A

def DRSv3_get_calib_data(archive, file_rad, keywords, properties, fiber='A', order_selection=None):

    calib_dict = {}

    selected_orders = DRSv3_give_back_selected_orders(properties, fiber, order_selection)

    map_orders_A, map_orders_B = DRSv3_map_orders_AB(properties, order_selection)

    if fiber=='A':
        calib_dict['fibAB_orders_match'] = map_orders_A - np.min(selected_orders)
        """ The first selected order could not have a match in fiber B, so we need to renumber from the first order of
            the input selection, not from the first order that had a match """
    else:
        calib_dict['fibAB_orders_match'] = map_orders_B - np.min(map_orders_B)
        """ Only the orders with a match in fiber A are read in the first place, so we can safely rescale with respect
            to the number of the first order in the matched list """

    e2ds_fits = fits.open(archive+'/'+file_rad+'_e2ds_'+fiber+'.fits')

    if e2ds_fits[0].header[keywords['header_dpr_catg']] != 'SCIENCE':
        return

    try:
        blaze_file = e2ds_fits[0].header[keywords['header_blaze']]
        blaze_fits = fits.open(archive + '/' + blaze_file)
    except:
        blaze_file = e2ds_fits[0].header[keywords['header_blaze']].replace(':', '-')
        blaze_fits = fits.open(archive + '/' + blaze_file)

    # getting blaze file
    calib_dict['blaze'] = blaze_fits[0].data[selected_orders, :]

    # getting lamp file
    try:
        lamp_fits = fits.open(archive + '/' + blaze_file[:29] + '_lamp_' + fiber + '.fits')
        calib_dict['lamp'] = lamp_fits[0].data[selected_orders, :]
        lamp_fits.close()
    except:
        print("lamp files not available, sky correction will not be performed")

    calib_dict['n_pixels'] = blaze_fits[0].header['NAXIS1']
    calib_dict['n_orders'] = len(selected_orders)

    blaze_fits.close()

    return calib_dict


def DRSv3_get_input_data(archive, file_rad, keywords, properties, mask, fiber='A', skip_ccf=None, skip_s1d=True, order_selection=None):

    input_dict = {'mask': mask, 'header':{}}
    input_s1d = {'header':{}}

    selected_orders = DRSv3_give_back_selected_orders(properties, fiber, order_selection)

    if mask is None:
        skip_ccf = True

    e2ds_fits = fits.open(archive+'/'+file_rad+'_e2ds_'+fiber+'.fits')
    input_dict['header']['e2ds'] = e2ds_fits[0].header

    input_dict['n_pixels'] = e2ds_fits[0].header['NAXIS1']
    input_dict['n_orders'] = len(selected_orders)

    input_dict['DPR_CATG'] = e2ds_fits[0].header[keywords['header_dpr_catg']]

    if input_dict['DPR_CATG'] != 'SCIENCE':
        return
    input_dict['DPR_TYPE'] = e2ds_fits[0].header[keywords['header_dpr_type']]

    if not skip_s1d:
        s1d_fits = fits.open(archive + '/' + file_rad + '_s1d_'+fiber+'.fits')
        input_dict['header']['s1d'] = s1d_fits[0].header
        input_s1d['header']['s1d'] = s1d_fits[0].header

        temp_wave, temp_step = DRSv3_get_s1d_wave(s1d_fits)
        sel_wave = (temp_wave >= 3879.99990) & (temp_wave <= 6900.0001)

        input_s1d['flux'] = s1d_fits[0].data[sel_wave]
        input_s1d['wave'] = temp_wave[sel_wave]
        input_s1d['step'] = temp_step[sel_wave]
        input_s1d['size'] = np.size(input_s1d['wave'])

        s1d_fits.close()

    if not skip_ccf:
        ccf_fits = fits.open(archive+'/'+file_rad+'_ccf_'+mask+'_'+fiber+'.fits')
        input_dict['RVC'] = ccf_fits[0].header[keywords['header_rvc']]
        input_dict['header']['ccf'] = ccf_fits[0].header



    input_dict['BERV'] = e2ds_fits[0].header[keywords['header_berv']]
    input_dict['EXPTIME'] = e2ds_fits[0].header['EXPTIME']

    if properties['time_stamp'] == 'start_exposure':
        input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']] + input_dict['EXPTIME']/86400.
        input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']] + input_dict['EXPTIME']/86400.
    elif properties['time_stamp'] == 'mid_exposure':
        input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']]
        input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']]
    elif properties['time_stamp'] == 'end_exposure':
        input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']] - input_dict['EXPTIME']/86400.
        input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']] - input_dict['EXPTIME']/86400.
    else:
        print('*** please specify the relationship between epoch and exposure time - assuming mid-exposure epochs')
        input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']]
        input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']]

    if properties['time_standard'] == 'UTC':
        input_dict['BJD']+= difference_utc2tdb(input_dict['MJD']+2400000.5)


    input_dict['LST'] = e2ds_fits[0].header['LST']

    try:
        input_dict['AIRMASS'] = e2ds_fits[0].header['AIRMASS']
    except:
        input_dict['AIRMASS'] = (e2ds_fits[0].header[keywords['airmass_alt_start']]
									+ e2ds_fits[0].header[keywords['airmass_alt_end']])/2.

    try:
        input_dict['AIRM_START'] = e2ds_fits[0].header[keywords['airmass_alt_start']]
        input_dict['AIRM_END'] = e2ds_fits[0].header[keywords['airmass_alt_end']]
    except:
        input_dict['AIRM_START'] = input_dict['AIRMASS'] -0.05
        input_dict['AIRM_END'] = max(1.00, input_dict['AIRMASS'] +0.05)

    input_dict['UTC'] = (input_dict['MJD'] - int(input_dict['MJD'])) * 86400.
    input_dict['HUMIDITY'] = e2ds_fits[0].header[keywords['humidity']]
    input_dict['PRESSURE'] = e2ds_fits[0].header[keywords['pressure']]
    input_dict['TEMPERATURE_EN'] = e2ds_fits[0].header[keywords['temperature_env']]
    input_dict['TEMPERATURE_M1'] = e2ds_fits[0].header[keywords['temperature_m1']]
    input_dict['ELEVATION'] = np.arcsin(1./input_dict['AIRMASS']) * (180./np.pi)

    input_dict['GEOELEV'] = properties['geoelev']
    input_dict['GEOLONG'] = properties['longitude']
    input_dict['GEOLAT'] = properties['latitude']

    input_dict['molecfit'] = properties['molecfit']

    try:
        try:
            input_dict['RA'] = e2ds_fits[0].header['RA-DEG']
            input_dict['DEC'] = e2ds_fits[0].header['DEC-DEG']
        except:
            input_dict['RA'] = e2ds_fits[0].header['RA-RAD'] * 180.00 / np.pi
            input_dict['DEC'] = e2ds_fits[0].header['DEC-RAD'] * 180.00 / np.pi  # weird choice of using DEC in hours
    except:
            input_dict['RA'] = e2ds_fits[0].header['RA']
            input_dict['DEC'] = e2ds_fits[0].header['DEC']

    try:
        input_dict['BERV'] = e2ds_fits[0].header[keywords['header_berv']]
        input_dict['EXPTIME'] = e2ds_fits[0].header['EXPTIME']

        if properties['time_stamp'] == 'start_exposure':
            input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']] + input_dict['EXPTIME']/86400.
            input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']] + input_dict['EXPTIME']/86400.
        elif properties['time_stamp'] == 'mid_exposure':
            input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']]
            input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']]
        elif properties['time_stamp'] == 'end_exposure':
            input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']] - input_dict['EXPTIME']/86400.
            input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']] - input_dict['EXPTIME']/86400.
        else:
            print('*** please specify the relationship between epoch and exposure time - assuming mid-exposure epochs')
            input_dict['BJD'] = e2ds_fits[0].header[keywords['header_bjd']]
            input_dict['MJD'] = e2ds_fits[0].header[keywords['header_mjd']]

        if properties['time_standard'] == 'UTC':
            input_dict['BJD']+= difference_utc2tdb(input_dict['MJD']+2400000.5)


        input_dict['LST'] = e2ds_fits[0].header['LST']

        try:
            input_dict['AIRMASS'] = e2ds_fits[0].header['AIRMASS']
        except:
            input_dict['AIRMASS'] = (e2ds_fits[0].header[keywords['airmass_alt_start']]
										+ e2ds_fits[0].header[keywords['airmass_alt_end']])/2.

        input_dict['UTC'] = (input_dict['MJD'] - int(input_dict['MJD'])) * 86400.
        input_dict['HUMIDITY'] = e2ds_fits[0].header[keywords['humidity']]
        input_dict['PRESSURE'] = e2ds_fits[0].header[keywords['pressure']]
        input_dict['TEMPERATURE_EN'] = e2ds_fits[0].header[keywords['temperature_env']]
        input_dict['TEMPERATURE_M1'] = e2ds_fits[0].header[keywords['temperature_m1']]
        input_dict['ELEVATION'] = np.arcsin(1./input_dict['AIRMASS']) * (180./np.pi)

        input_dict['GEOELEV'] = properties['geoelev']
        input_dict['GEOLONG'] = properties['longitude']
        input_dict['GEOLAT'] = properties['latitude']

        input_dict['molecfit'] = properties['molecfit']

        try:
            try:
                input_dict['RA'] = e2ds_fits[0].header['RA-DEG']
                input_dict['DEC'] = e2ds_fits[0].header['DEC-DEG']
            except:
                input_dict['RA'] = e2ds_fits[0].header['RA-RAD'] * 180.00 / np.pi
                input_dict['DEC'] = e2ds_fits[0].header['DEC-RAD'] * 180.00 / np.pi  # weird choice of using DEC in hours
        except:
                input_dict['RA'] = e2ds_fits[0].header['RA']
                input_dict['DEC'] = e2ds_fits[0].header['DEC']

    except:
        print('Keyword error in prepare_dataset - check the FITS header of your files')
        quit()
        pass

    input_dict['BLAZE_file'] = e2ds_fits[0].header[keywords['header_blaze']]
    input_dict['CCD_SIGDET'] = e2ds_fits[0].header[keywords['header_ccd']]
    input_dict['CCD_GAIN'] = e2ds_fits[0].header[keywords['header_conad']]

    # getting data
    input_dict['e2ds'] = e2ds_fits[0].data[selected_orders, :]
    input_dict['e2ds_err'] = np.sqrt(np.abs(input_dict['e2ds']))

    temp_wave, temp_step = DRSv3_get_e2ds_wave(e2ds_fits, keywords['header_deg_ll'], keywords['header_coeff_ll'])

    input_dict['wave'] = temp_wave[selected_orders, :]
    input_dict['step'] = temp_step[selected_orders, :]
    input_dict['orders'] = len(selected_orders)

    input_dict['wave_size'] = e2ds_fits[0].header['NAXIS1']

    e2ds_fits.close()
    if not skip_ccf:
        ccf_fits.close()

    return input_dict,input_s1d


def DRSv3_get_s1d_wave(s1d_fits):
    return np.arange(0, s1d_fits[0].header['NAXIS1'], 1.)*s1d_fits[0].header['CDELT1'] + s1d_fits[0].header['CRVAL1'], \
           np.ones(s1d_fits[0].header['NAXIS1'])*s1d_fits[0].header['CDELT1']


def DRSv3_get_e2ds_wave(e2ds_fits, header_deg_ll, header_coeff_ll, order=None):

    e2ds_o = e2ds_fits[0].header['NAXIS2']
    e2ds_w = e2ds_fits[0].header['NAXIS1']

    e2ds_wave = np.zeros([e2ds_o, e2ds_w], dtype=np.double)
    e2ds_step = np.zeros([e2ds_o, e2ds_w], dtype=np.double)

    d = e2ds_fits[0].header[header_deg_ll]
    x = np.arange(0, e2ds_w, 1.)

    for n in range(0, e2ds_o):
        for i in range(d, -1, -1):
            a_sel = i + n*(1+d)
            a_coeff = e2ds_fits[0].header[header_coeff_ll+repr(a_sel)]
            if i == d:
                y_w = a_coeff
                y_s = i*a_coeff
            else:
                y_w = y_w*x + a_coeff
                if i > 0: y_s = y_s*x + i*a_coeff
        e2ds_wave[n, :] = y_w
        e2ds_step[n, :] = y_s

    if order is None:
        return e2ds_wave, e2ds_step
    else:
        return e2ds_wave[order, :], e2ds_step[order, :]
